/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1998-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions required by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
#ifndef plhash_h___^M
#define plhash_h___^M
/*^M
 * API to portable hash table code.^M
 */^M
#include <stdio.h>^M
#include "prtypes.h"^M
^M
PR_BEGIN_EXTERN_C^M
^M
typedef struct PLHashEntry  PLHashEntry;^M
typedef struct PLHashTable  PLHashTable;^M
typedef PRUint32 PLHashNumber;^M
#define PL_HASH_BITS 32  /* Number of bits in PLHashNumber */^M
typedef PLHashNumber (PR_CALLBACK *PLHashFunction)(const void *key);^M
typedef PRIntn (PR_CALLBACK *PLHashComparator)(const void *v1, const void *v2);^M
^M
#if defined(XP_OS2_VACPP) && defined(VACPP_FLIP) /* for nsSpaceManager.cpp */^M
PR_END_EXTERN_C                                  /* and nsHTMLDocument.cpp */^M
#endif^M
typedef PRIntn (PR_CALLBACK *PLHashEnumerator)(PLHashEntry *he, PRIntn i, void *arg);^M
^M
#if defined(XP_OS2_VACPP) && defined(VACPP_FLIP)^M
PR_BEGIN_EXTERN_C^M
#endif^M
^M
/* Flag bits in PLHashEnumerator's return value */^M
#define HT_ENUMERATE_NEXT       0       /* continue enumerating entries */^M
#define HT_ENUMERATE_STOP       1       /* stop enumerating entries */^M
#define HT_ENUMERATE_REMOVE     2       /* remove and free the current entry */^M
#define HT_ENUMERATE_UNHASH     4       /* just unhash the current entry */^M
^M
typedef struct PLHashAllocOps {^M
    void *              (PR_CALLBACK *allocTable)(void *pool, PRSize size);^M
    void                (PR_CALLBACK *freeTable)(void *pool, void *item);^M
    PLHashEntry *       (PR_CALLBACK *allocEntry)(void *pool, const void *key);^M
    void                (PR_CALLBACK *freeEntry)(void *pool, PLHashEntry *he, PRUintn flag);^M
} PLHashAllocOps;^M
^M
#define HT_FREE_VALUE   0               /* just free the entry's value */^M
#define HT_FREE_ENTRY   1               /* free value and entire entry */^M
^M
struct PLHashEntry {^M
    PLHashEntry         *next;          /* hash chain linkage */^M
    PLHashNumber        keyHash;        /* key hash function result */^M
    const void          *key;           /* ptr to opaque key */^M
    void                *value;         /* ptr to opaque value */^M
};^M
^M
struct PLHashTable {^M
    PLHashEntry         **buckets;      /* vector of hash buckets */^M
    PRUint32              nentries;       /* number of entries in table */^M
    PRUint32              shift;          /* multiplicative hash shift */^M
    PLHashFunction      keyHash;        /* key hash function */^M
    PLHashComparator    keyCompare;     /* key comparison function */^M
    PLHashComparator    valueCompare;   /* value comparison function */^M
    const PLHashAllocOps *allocOps;     /* allocation operations */^M
    void                *allocPriv;     /* allocation private data */^M
#ifdef HASHMETER^M
    PRUint32              nlookups;       /* total number of lookups */^M
    PRUint32              nsteps;         /* number of hash chains traversed */^M
    PRUint32              ngrows;         /* number of table expansions */^M
    PRUint32              nshrinks;       /* number of table contractions */^M
#endif^M
};^M
^M
/*^M
 * Create a new hash table.^M
 * If allocOps is null, use default allocator ops built on top of malloc().^M
 */^M
PR_EXTERN(PLHashTable *)^M
PL_NewHashTable(PRUint32 numBuckets, PLHashFunction keyHash,^M
                PLHashComparator keyCompare, PLHashComparator valueCompare,^M
                const PLHashAllocOps *allocOps, void *allocPriv);^M
^M
PR_EXTERN(void)^M
PL_HashTableDestroy(PLHashTable *ht);^M
^M
/* Higher level access methods */^M
PR_EXTERN(PLHashEntry *)^M
PL_HashTableAdd(PLHashTable *ht, const void *key, void *value);^M
^M
PR_EXTERN(PRBool)^M
PL_HashTableRemove(PLHashTable *ht, const void *key);^M
^M
PR_EXTERN(void *)^M
PL_HashTableLookup(PLHashTable *ht, const void *key);^M
^M
PR_EXTERN(void *)^M
PL_HashTableLookupConst(PLHashTable *ht, const void *key);^M
^M
PR_EXTERN(PRIntn)^M
PL_HashTableEnumerateEntries(PLHashTable *ht, PLHashEnumerator f, void *arg);^M
^M
/* General-purpose C string hash function. */^M
PR_EXTERN(PLHashNumber)^M
PL_HashString(const void *key);^M
^M
/* Compare strings using strcmp(), return true if equal. */^M
PR_EXTERN(PRIntn)^M
PL_CompareStrings(const void *v1, const void *v2);^M
^M
/* Stub function just returns v1 == v2 */^M
PR_EXTERN(PRIntn)^M
PL_CompareValues(const void *v1, const void *v2);^M
^M
/* Low level access methods */^M
PR_EXTERN(PLHashEntry **)^M
PL_HashTableRawLookup(PLHashTable *ht, PLHashNumber keyHash, const void *key);^M
^M
PR_EXTERN(PLHashEntry **)^M
PL_HashTableRawLookupConst(PLHashTable *ht, PLHashNumber keyHash,^M
                           const void *key);^M
^M
PR_EXTERN(PLHashEntry *)^M
PL_HashTableRawAdd(PLHashTable *ht, PLHashEntry **hep, PLHashNumber keyHash,^M
                   const void *key, void *value);^M
^M
PR_EXTERN(void)^M
PL_HashTableRawRemove(PLHashTable *ht, PLHashEntry **hep, PLHashEntry *he);^M
^M
/* This can be trivially implemented using PL_HashTableEnumerateEntries. */^M
PR_EXTERN(PRIntn)^M
PL_HashTableDump(PLHashTable *ht, PLHashEnumerator dump, FILE *fp);^M
^M
PR_END_EXTERN_C^M
^M
#endif /* plhash_h___ */^M
