/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1998-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions required by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
/*^M
 * File:     prio.h^M
 *^M
 * Description:    PR i/o related stuff, such as file system access, file^M
 *         i/o, socket i/o, etc.^M
 */^M
^M
#ifndef prio_h___^M
#define prio_h___^M
^M
#include "prlong.h"^M
#include "prtime.h"^M
#include "prinrval.h"^M
#include "prinet.h"^M
^M
PR_BEGIN_EXTERN_C^M
^M
/* Typedefs */^M
typedef struct PRDir            PRDir;^M
typedef struct PRDirEntry       PRDirEntry;^M
typedef struct PRFileDesc       PRFileDesc;^M
typedef struct PRFileInfo       PRFileInfo;^M
typedef struct PRFileInfo64     PRFileInfo64;^M
typedef union  PRNetAddr        PRNetAddr;^M
typedef struct PRIOMethods      PRIOMethods;^M
typedef struct PRPollDesc       PRPollDesc;^M
typedef struct PRFilePrivate    PRFilePrivate;^M
typedef struct PRSendFileData   PRSendFileData;^M
^M
/*^M
***************************************************************************^M
** The file descriptor.^M
** This is the primary structure to represent any active open socket,^M
** whether it be a normal file or a network connection. Such objects^M
** are stackable (or layerable). Each layer may have its own set of^M
** method pointers and context private to that layer. All each layer^M
** knows about its neighbors is how to get to their method table.^M
***************************************************************************^M
*/^M
^M
typedef PRIntn PRDescIdentity;          /* see: Layering file descriptors */^M
^M
struct PRFileDesc {^M
    const PRIOMethods *methods;         /* the I/O methods table */^M
    PRFilePrivate *secret;              /* layer dependent data */^M
    PRFileDesc *lower, *higher;         /* pointers to adjacent layers */^M
    void (PR_CALLBACK *dtor)(PRFileDesc *fd);^M
                                        /* A destructor function for layer */^M
    PRDescIdentity identity;            /* Identity of this particular layer  */^M
};^M
^M
/*^M
***************************************************************************^M
** PRTransmitFileFlags^M
**^M
** Flags for PR_TransmitFile.  Pass PR_TRANSMITFILE_CLOSE_SOCKET to^M
** PR_TransmitFile if the connection should be closed after the file^M
** is transmitted.^M
***************************************************************************^M
*/^M
typedef enum PRTransmitFileFlags {^M
    PR_TRANSMITFILE_KEEP_OPEN = 0,    /* socket is left open after file^M
                                       * is transmitted. */^M
    PR_TRANSMITFILE_CLOSE_SOCKET = 1  /* socket is closed after file^M
                                       * is transmitted. */^M
} PRTransmitFileFlags;^M
^M
/*^M
**************************************************************************^M
** Macros for PRNetAddr^M
**^M
** Address families: PR_AF_INET, PR_AF_INET6, PR_AF_LOCAL^M
** IP addresses: PR_INADDR_ANY, PR_INADDR_LOOPBACK, PR_INADDR_BROADCAST^M
**************************************************************************^M
*/^M
^M
#ifdef WIN32^M
^M
#define PR_AF_INET 2^M
#define PR_AF_LOCAL 1^M
#define PR_INADDR_ANY (unsigned long)0x00000000^M
#define PR_INADDR_LOOPBACK 0x7f000001^M
#define PR_INADDR_BROADCAST (unsigned long)0xffffffff^M
^M
#else /* WIN32 */^M
^M
#define PR_AF_INET AF_INET^M
#define PR_AF_LOCAL AF_UNIX^M
#define PR_INADDR_ANY INADDR_ANY^M
#define PR_INADDR_LOOPBACK INADDR_LOOPBACK^M
#define PR_INADDR_BROADCAST INADDR_BROADCAST^M
^M
#endif /* WIN32 */^M
^M
/*^M
** Define PR_AF_INET6 in prcpucfg.h with the same^M
** value as AF_INET6 on platforms with IPv6 support.^M
** Otherwise define it here.^M
*/^M
#ifndef PR_AF_INET6^M
#define PR_AF_INET6 100^M
#endif^M
^M
/*^M
**************************************************************************^M
** A network address^M
**^M
** Only Internet Protocol (IPv4 and IPv6) addresses are supported.^M
** The address family must always represent IPv4 (AF_INET, probably == 2)^M
** or IPv6 (AF_INET6).^M
**************************************************************************^M
*************************************************************************/^M
^M
struct PRIPv6Addr {^M
	union {^M
		PRUint8  _S6_u8[16];^M
		PRUint16 _S6_u16[8];^M
		PRUint32 _S6_u32[4];^M
		PRUint64 _S6_u64[2];^M
	} _S6_un;^M
};^M
#define pr_s6_addr		_S6_un._S6_u8^M
#define pr_s6_addr16	_S6_un._S6_u16^M
#define pr_s6_addr32	_S6_un._S6_u32^M
#define pr_s6_addr64 	_S6_un._S6_addr64^M
^M
typedef struct PRIPv6Addr PRIPv6Addr;^M
^M
union PRNetAddr {^M
    struct {^M
        PRUint16 family;                /* address family (0x00ff maskable) */^M
#ifdef XP_BEOS^M
        char data[10];                  /* Be has a smaller structure */^M
#else^M
        char data[14];                  /* raw address data */^M
#endif^M
    } raw;^M
    struct {^M
        PRUint16 family;                /* address family (AF_INET) */^M
        PRUint16 port;                  /* port number */^M
        PRUint32 ip;                    /* The actual 32 bits of address */^M
#ifdef XP_BEOS^M
        char pad[4];                    /* Be has a smaller structure */^M
#else^M
        char pad[8];^M
#endif^M
    } inet;^M
    struct {^M
        PRUint16 family;                /* address family (AF_INET6) */^M
        PRUint16 port;                  /* port number */^M
        PRUint32 flowinfo;              /* routing information */^M
        PRIPv6Addr ip;                  /* the actual 128 bits of address */^M
        PRUint32 scope_id;              /* set of interfaces for a scope */^M
    } ipv6;^M
#if defined(XP_UNIX)^M
    struct {                            /* Unix domain socket address */^M
        PRUint16 family;                /* address family (AF_UNIX) */^M
        char path[104];                 /* null-terminated pathname */^M
    } local;^M
#endif^M
};^M
^M
/*^M
***************************************************************************^M
** PRSockOption^M
**^M
** The file descriptors can have predefined options set after they file^M
** descriptor is created to change their behavior. Only the options in^M
** the following enumeration are supported.^M
***************************************************************************^M
*/^M
typedef enum PRSockOption^M
{^M
    PR_SockOpt_Nonblocking,     /* nonblocking io */^M
    PR_SockOpt_Linger,          /* linger on close if data present */^M
    PR_SockOpt_Reuseaddr,       /* allow local address reuse */^M
    PR_SockOpt_Keepalive,       /* keep connections alive */^M
    PR_SockOpt_RecvBufferSize,  /* send buffer size */^M
    PR_SockOpt_SendBufferSize,  /* receive buffer size */^M
^M
    PR_SockOpt_IpTimeToLive,    /* time to live */^M
    PR_SockOpt_IpTypeOfService, /* type of service and precedence */^M
^M
    PR_SockOpt_AddMember,       /* add an IP group membership */^M
    PR_SockOpt_DropMember,      /* drop an IP group membership */^M
    PR_SockOpt_McastInterface,  /* multicast interface address */^M
    PR_SockOpt_McastTimeToLive, /* multicast timetolive */^M
    PR_SockOpt_McastLoopback,   /* multicast loopback */^M
^M
    PR_SockOpt_NoDelay,         /* don't delay send to coalesce packets */^M
    PR_SockOpt_MaxSegment,      /* maximum segment size */^M
    PR_SockOpt_Broadcast,       /* enable broadcast */^M
    PR_SockOpt_Last^M
} PRSockOption;^M
^M
typedef struct PRLinger {^M
	PRBool polarity;		    /* Polarity of the option's setting */^M
	PRIntervalTime linger;	    /* Time to linger before closing */^M
} PRLinger;^M
^M
typedef struct PRMcastRequest {^M
	PRNetAddr mcaddr;			/* IP multicast address of group */^M
	PRNetAddr ifaddr;			/* local IP address of interface */^M
} PRMcastRequest;^M
^M
typedef struct PRSocketOptionData^M
{^M
    PRSockOption option;^M
    union^M
    {^M
        PRUintn ip_ttl;             /* IP time to live */^M
        PRUintn mcast_ttl;          /* IP multicast time to live */^M
        PRUintn tos;                /* IP type of service and precedence */^M
        PRBool non_blocking;        /* Non-blocking (network) I/O */^M
        PRBool reuse_addr;          /* Allow local address reuse */^M
        PRBool keep_alive;          /* Keep connections alive */^M
        PRBool mcast_loopback;      /* IP multicast loopback */^M
        PRBool no_delay;            /* Don't delay send to coalesce packets */^M
        PRBool broadcast;           /* Enable broadcast */^M
        PRSize max_segment;         /* Maximum segment size */^M
        PRSize recv_buffer_size;    /* Receive buffer size */^M
        PRSize send_buffer_size;    /* Send buffer size */^M
        PRLinger linger;            /* Time to linger on close if data present */^M
        PRMcastRequest add_member;  /* add an IP group membership */^M
        PRMcastRequest drop_member; /* Drop an IP group membership */^M
        PRNetAddr mcast_if;         /* multicast interface address */^M
    } value;^M
} PRSocketOptionData;^M
^M
/*^M
***************************************************************************^M
** PRIOVec^M
**^M
** The I/O vector is used by the write vector method to describe the areas^M
** that are affected by the ouput operation.^M
***************************************************************************^M
*/^M
typedef struct PRIOVec {^M
    char *iov_base;^M
    int iov_len;^M
} PRIOVec;^M
^M
/*^M
***************************************************************************^M
** Discover what type of socket is being described by the file descriptor.^M
***************************************************************************^M
*/^M
typedef enum PRDescType^M
{^M
    PR_DESC_FILE = 1,^M
    PR_DESC_SOCKET_TCP = 2,^M
    PR_DESC_SOCKET_UDP = 3,^M
    PR_DESC_LAYERED = 4,^M
    PR_DESC_PIPE = 5^M
} PRDescType;^M
^M
typedef enum PRSeekWhence {^M
    PR_SEEK_SET = 0,^M
    PR_SEEK_CUR = 1,^M
    PR_SEEK_END = 2^M
} PRSeekWhence;^M
^M
NSPR_API(PRDescType) PR_GetDescType(PRFileDesc *file);^M
^M
/*^M
***************************************************************************^M
** PRIOMethods^M
**^M
** The I/O methods table provides procedural access to the functions of^M
** the file descriptor. It is the responsibility of a layer implementor^M
** to provide suitable functions at every entry point. If a layer provides^M
** no functionality, it should call the next lower(higher) function of the^M
** same name (e.g., return fd->lower->method->close(fd->lower));^M
**^M
** Not all functions are implemented for all types of files. In cases where^M
** that is true, the function will return a error indication with an error^M
** code of PR_INVALID_METHOD_ERROR.^M
***************************************************************************^M
*/^M
^M
typedef PRStatus (PR_CALLBACK *PRCloseFN)(PRFileDesc *fd);^M
typedef PRInt32 (PR_CALLBACK *PRReadFN)(PRFileDesc *fd, void *buf, PRInt32 amount);^M
typedef PRInt32 (PR_CALLBACK *PRWriteFN)(PRFileDesc *fd, const void *buf, PRInt32 amount);^M
typedef PRInt32 (PR_CALLBACK *PRAvailableFN)(PRFileDesc *fd);^M
typedef PRInt64 (PR_CALLBACK *PRAvailable64FN)(PRFileDesc *fd);^M
typedef PRStatus (PR_CALLBACK *PRFsyncFN)(PRFileDesc *fd);^M
typedef PROffset32 (PR_CALLBACK *PRSeekFN)(PRFileDesc *fd, PROffset32 offset, PRSeekWhence how);^M
typedef PROffset64 (PR_CALLBACK *PRSeek64FN)(PRFileDesc *fd, PROffset64 offset, PRSeekWhence how);^M
typedef PRStatus (PR_CALLBACK *PRFileInfoFN)(PRFileDesc *fd, PRFileInfo *info);^M
typedef PRStatus (PR_CALLBACK *PRFileInfo64FN)(PRFileDesc *fd, PRFileInfo64 *info);^M
typedef PRInt32 (PR_CALLBACK *PRWritevFN)(^M
    PRFileDesc *fd, const PRIOVec *iov, PRInt32 iov_size,^M
    PRIntervalTime timeout);^M
typedef PRStatus (PR_CALLBACK *PRConnectFN)(^M
    PRFileDesc *fd, const PRNetAddr *addr, PRIntervalTime timeout);^M
typedef PRFileDesc* (PR_CALLBACK *PRAcceptFN) (^M
    PRFileDesc *fd, PRNetAddr *addr, PRIntervalTime timeout);^M
typedef PRStatus (PR_CALLBACK *PRBindFN)(PRFileDesc *fd, const PRNetAddr *addr);^M
typedef PRStatus (PR_CALLBACK *PRListenFN)(PRFileDesc *fd, PRIntn backlog);^M
typedef PRStatus (PR_CALLBACK *PRShutdownFN)(PRFileDesc *fd, PRIntn how);^M
typedef PRInt32 (PR_CALLBACK *PRRecvFN)(^M
    PRFileDesc *fd, void *buf, PRInt32 amount,^M
    PRIntn flags, PRIntervalTime timeout);^M
typedef PRInt32 (PR_CALLBACK *PRSendFN) (^M
    PRFileDesc *fd, const void *buf, PRInt32 amount,^M
    PRIntn flags, PRIntervalTime timeout);^M
typedef PRInt32 (PR_CALLBACK *PRRecvfromFN)(^M
    PRFileDesc *fd, void *buf, PRInt32 amount,^M
    PRIntn flags, PRNetAddr *addr, PRIntervalTime timeout);^M
typedef PRInt32 (PR_CALLBACK *PRSendtoFN)(^M
    PRFileDesc *fd, const void *buf, PRInt32 amount,^M
    PRIntn flags, const PRNetAddr *addr, PRIntervalTime timeout);^M
typedef PRInt16 (PR_CALLBACK *PRPollFN)(^M
    PRFileDesc *fd, PRInt16 in_flags, PRInt16 *out_flags);^M
typedef PRInt32 (PR_CALLBACK *PRAcceptreadFN)(^M
    PRFileDesc *sd, PRFileDesc **nd, PRNetAddr **raddr,^M
    void *buf, PRInt32 amount, PRIntervalTime t);^M
typedef PRInt32 (PR_CALLBACK *PRTransmitfileFN)(^M
     PRFileDesc *sd, PRFileDesc *fd, const void *headers,^M
     PRInt32 hlen, PRTransmitFileFlags flags, PRIntervalTime t);^M
typedef PRStatus (PR_CALLBACK *PRGetsocknameFN)(PRFileDesc *fd, PRNetAddr *addr);^M
typedef PRStatus (PR_CALLBACK *PRGetpeernameFN)(PRFileDesc *fd, PRNetAddr *addr);^M
typedef PRStatus (PR_CALLBACK *PRGetsocketoptionFN)(^M
    PRFileDesc *fd, PRSocketOptionData *data);^M
typedef PRStatus (PR_CALLBACK *PRSetsocketoptionFN)(^M
    PRFileDesc *fd, const PRSocketOptionData *data);^M
typedef PRInt32 (PR_CALLBACK *PRSendfileFN)(^M
	PRFileDesc *networkSocket, PRSendFileData *sendData,^M
	PRTransmitFileFlags flags, PRIntervalTime timeout);^M
typedef PRStatus (PR_CALLBACK *PRConnectcontinueFN)(^M
    PRFileDesc *fd, PRInt16 out_flags);^M
typedef PRIntn (PR_CALLBACK *PRReservedFN)(PRFileDesc *fd);^M
^M
struct PRIOMethods {^M
    PRDescType file_type;           /* Type of file represented (tos)           */^M
    PRCloseFN close;                /* close file and destroy descriptor        */^M
    PRReadFN read;                  /* read up to specified bytes into buffer   */^M
    PRWriteFN write;                /* write specified bytes from buffer        */^M
    PRAvailableFN available;        /* determine number of bytes available      */^M
    PRAvailable64FN available64;    /*          ditto, 64 bit                   */^M
    PRFsyncFN fsync;                /* flush all buffers to permanent store     */^M
    PRSeekFN seek;                  /* position the file to the desired place   */^M
    PRSeek64FN seek64;              /*           ditto, 64 bit                  */^M
    PRFileInfoFN fileInfo;          /* Get information about an open file       */^M
    PRFileInfo64FN fileInfo64;      /*           ditto, 64 bit                  */^M
    PRWritevFN writev;              /* Write segments as described by iovector  */^M
    PRConnectFN connect;            /* Connect to the specified (net) address   */^M
    PRAcceptFN accept;              /* Accept a connection for a (net) peer     */^M
    PRBindFN bind;                  /* Associate a (net) address with the fd    */^M
    PRListenFN listen;              /* Prepare to listen for (net) connections  */^M
    PRShutdownFN shutdown;          /* Shutdown a (net) connection              */^M
    PRRecvFN recv;                  /* Solicit up the the specified bytes       */^M
    PRSendFN send;                  /* Send all the bytes specified             */^M
    PRRecvfromFN recvfrom;          /* Solicit (net) bytes and report source    */^M
    PRSendtoFN sendto;              /* Send bytes to (net) address specified    */^M
    PRPollFN poll;                  /* Test the fd to see if it is ready        */^M
    PRAcceptreadFN acceptread;      /* Accept and read on a new (net) fd        */^M
    PRTransmitfileFN transmitfile;  /* Transmit at entire file                  */^M
    PRGetsocknameFN getsockname;    /* Get (net) address associated with fd     */^M
    PRGetpeernameFN getpeername;    /* Get peer's (net) address                 */^M
    PRReservedFN reserved_fn_6;     /* reserved for future use */^M
    PRReservedFN reserved_fn_5;     /* reserved for future use */^M
    PRGetsocketoptionFN getsocketoption;^M
                                    /* Get current setting of specified option  */^M
    PRSetsocketoptionFN setsocketoption;^M
                                    /* Set value of specified option            */^M
    PRSendfileFN sendfile;			/* Send a (partial) file with header/trailer*/^M
    PRConnectcontinueFN connectcontinue;^M
                                    /* Continue a nonblocking connect */^M
    PRReservedFN reserved_fn_3;		/* reserved for future use */^M
    PRReservedFN reserved_fn_2;		/* reserved for future use */^M
    PRReservedFN reserved_fn_1;		/* reserved for future use */^M
    PRReservedFN reserved_fn_0;		/* reserved for future use */^M
};^M
^M
/*^M
 **************************************************************************^M
 * FUNCTION: PR_GetSpecialFD^M
 * DESCRIPTION: Get the file descriptor that represents the standard input,^M
 *              output, or error stream.^M
 * INPUTS:^M
 *     PRSpecialFD id^M
 *         A value indicating the type of stream desired:^M
 *             PR_StandardInput: standard input^M
 *             PR_StandardOuput: standard output^M
 *             PR_StandardError: standard error^M
 * OUTPUTS: none^M
 * RETURNS: PRFileDesc *^M
 *     If the argument is valid, PR_GetSpecialFD returns a file descriptor^M
 *     that represents the corresponding standard I/O stream.  Otherwise,^M
 *     PR_GetSpecialFD returns NULL and sets error PR_INVALID_ARGUMENT_ERROR.^M
 **************************************************************************^M
 */^M
^M
typedef enum PRSpecialFD^M
{^M
    PR_StandardInput,          /* standard input */^M
    PR_StandardOutput,         /* standard output */^M
    PR_StandardError           /* standard error */^M
} PRSpecialFD;^M
^M
NSPR_API(PRFileDesc*) PR_GetSpecialFD(PRSpecialFD id);^M
^M
#define PR_STDIN	PR_GetSpecialFD(PR_StandardInput)^M
#define PR_STDOUT	PR_GetSpecialFD(PR_StandardOutput)^M
#define PR_STDERR	PR_GetSpecialFD(PR_StandardError)^M
^M
/*^M
 **************************************************************************^M
 * Layering file descriptors^M
 *^M
 * File descriptors may be layered. Each layer has it's own identity.^M
 * Identities are allocated by the runtime and are to be associated^M
 * (by the layer implementor) with all layers that are of that type.^M
 * It is then possible to scan the chain of layers and find a layer^M
 * that one recongizes and therefore predict that it will implement^M
 * a desired protocol.^M
 *^M
 * There are three well-known identities:^M
 *      PR_INVALID_IO_LAYER => an invalid layer identity, for error return^M
 *      PR_TOP_IO_LAYER     => the identity of the top of the stack^M
 *      PR_NSPR_IO_LAYER    => the identity used by NSPR proper^M
 * PR_TOP_IO_LAYER may be used as a shorthand for identifying the topmost^M
 * layer of an existing stack. Ie., the following two constructs are^M
 * equivalent.^M
 *^M
 *      rv = PR_PushIOLayer(stack, PR_TOP_IO_LAYER, my_layer);^M
 *      rv = PR_PushIOLayer(stack, PR_GetLayersIdentity(stack), my_layer)^M
 *^M
 * A string may be associated with the creation of the identity. It^M
 * will be copied by the runtime. If queried the runtime will return^M
 * a reference to that copied string (not yet another copy). There^M
 * is no facility for deleting an identity.^M
 **************************************************************************^M
 */^M
^M
#define PR_IO_LAYER_HEAD (PRDescIdentity)-3^M
#define PR_INVALID_IO_LAYER (PRDescIdentity)-1^M
#define PR_TOP_IO_LAYER (PRDescIdentity)-2^M
#define PR_NSPR_IO_LAYER (PRDescIdentity)0^M
^M
NSPR_API(PRDescIdentity) PR_GetUniqueIdentity(const char *layer_name);^M
NSPR_API(const char*) PR_GetNameForIdentity(PRDescIdentity ident);^M
NSPR_API(PRDescIdentity) PR_GetLayersIdentity(PRFileDesc* fd);^M
NSPR_API(PRFileDesc*) PR_GetIdentitiesLayer(PRFileDesc* fd_stack, PRDescIdentity id);^M
^M
/*^M
 **************************************************************************^M
 * PR_GetDefaultIOMethods: Accessing the default methods table.^M
 * You may get a pointer to the default methods table by calling this function.^M
 * You may then select any elements from that table with which to build your^M
 * layer's methods table. You may NOT modify the table directly.^M
 **************************************************************************^M
 */^M
NSPR_API(const PRIOMethods *) PR_GetDefaultIOMethods(void);^M
^M
/*^M
 **************************************************************************^M
 * Creating a layer^M
 *^M
 * A new layer may be allocated by calling PR_CreateIOLayerStub(). The^M
 * file descriptor returned will contain the pointer to the methods table^M
 * provided. The runtime will not modify the table nor test its correctness.^M
 **************************************************************************^M
 */^M
NSPR_API(PRFileDesc*) PR_CreateIOLayerStub(^M
    PRDescIdentity ident, const PRIOMethods *methods);^M
^M
/*^M
 **************************************************************************^M
 * Creating a layer^M
 *^M
 * A new stack may be created by calling PR_CreateIOLayer(). The^M
 * file descriptor returned will point to the top of the stack, which has^M
 * the layer 'fd' as the topmost layer.^M
 * ^M
 * NOTE: This function creates a new style stack, which has a fixed, dummy^M
 * header. The old style stack, created by a call to PR_PushIOLayer,^M
 * results in modifying contents of the top layer of the stack, when^M
 * pushing and popping layers of the stack.^M
 **************************************************************************^M
 */^M
NSPR_API(PRFileDesc*) PR_CreateIOLayer(PRFileDesc* fd);^M
^M
/*^M
 **************************************************************************^M
 * Pushing a layer^M
 *^M
 * A file descriptor (perhaps allocated using PR_CreateIOLayerStub()) may^M
 * be pushed into an existing stack of file descriptors at any point the^M
 * caller deems appropriate. The new layer will be inserted into the stack^M
 * just above the layer with the indicated identity.^M
 *^M
 * Note: Even if the identity parameter indicates the top-most layer of^M
 * the stack, the value of the file descriptor describing the original^M
 * stack will not change.^M
 **************************************************************************^M
 */^M
NSPR_API(PRStatus) PR_PushIOLayer(^M
    PRFileDesc *fd_stack, PRDescIdentity id, PRFileDesc *layer);^M
^M
/*^M
 **************************************************************************^M
 * Popping a layer^M
 *^M
 * A layer may be popped from a stack by indicating the identity of the^M
 * layer to be removed. If found, a pointer to the removed object will^M
 * be returned to the caller. The object then becomes the responsibility^M
 * of the caller.^M
 *^M
 * Note: Even if the identity indicates the top layer of the stack, the^M
 * reference returned will not be the file descriptor for the stack and^M
 * that file descriptor will remain valid.^M
 **************************************************************************^M
 */^M
NSPR_API(PRFileDesc*) PR_PopIOLayer(PRFileDesc *fd_stack, PRDescIdentity id);^M
^M
/*^M
 **************************************************************************^M
 * FUNCTION:    PR_Open^M
 * DESCRIPTION:    Open a file for reading, writing, or both.^M
 * INPUTS:^M
 *     const char *name^M
 *         The path name of the file to be opened^M
 *     PRIntn flags^M
 *         The file status flags.^M
 *         It is a bitwise OR of the following bit flags (only one of^M
 *         the first three flags below may be used):^M
 *		PR_RDONLY        Open for reading only.^M
 *		PR_WRONLY        Open for writing only.^M
 *		PR_RDWR          Open for reading and writing.^M
 *		PR_CREATE_FILE   If the file does not exist, the file is created^M
 *                              If the file exists, this flag has no effect.^M
 *      PR_SYNC          If set, each write will wait for both the file data^M
 *                              and file status to be physically updated.^M
 *		PR_APPEND        The file pointer is set to the end of^M
 *                              the file prior to each write.^M
 *		PR_TRUNCATE      If the file exists, its length is truncated to 0.^M
 *      PR_EXCL          With PR_CREATE_FILE, if the file does not exist,^M
 *                              the file is created. If the file already ^M
 *                              exists, no action and NULL is returned^M
 *^M
 *     PRIntn mode^M
 *         The access permission bits of the file mode, if the file is^M
 *         created when PR_CREATE_FILE is on.^M
 * OUTPUTS:    None^M
 * RETURNS:    PRFileDesc *^M
 *     If the file is successfully opened,^M
 *     returns a pointer to the PRFileDesc^M
 *     created for the newly opened file.^M
 *     Returns a NULL pointer if the open^M
 *     failed.^M
 * SIDE EFFECTS:^M
 * RESTRICTIONS:^M
 * MEMORY:^M
 *     The return value, if not NULL, points to a dynamically allocated^M
 *     PRFileDesc object.^M
 * ALGORITHM:^M
 **************************************************************************^M
 */^M
^M
/* Open flags */^M
#define PR_RDONLY       0x01^M
#define PR_WRONLY       0x02^M
#define PR_RDWR         0x04^M
#define PR_CREATE_FILE  0x08^M
#define PR_APPEND       0x10^M
#define PR_TRUNCATE     0x20^M
#define PR_SYNC         0x40^M
#define PR_EXCL         0x80^M
^M
/*^M
** File modes ....^M
**^M
** CAVEAT: 'mode' is currently only applicable on UNIX platforms.^M
** The 'mode' argument may be ignored by PR_Open on other platforms.^M
**^M
**   00400   Read by owner.^M
**   00200   Write by owner.^M
**   00100   Execute (search if a directory) by owner.^M
**   00040   Read by group.^M
**   00020   Write by group.^M
**   00010   Execute by group.^M
**   00004   Read by others.^M
**   00002   Write by others^M
**   00001   Execute by others.^M
**^M
*/^M
^M
NSPR_API(PRFileDesc*) PR_Open(const char *name, PRIntn flags, PRIntn mode);^M
^M
/*^M
 **************************************************************************^M
 * FUNCTION: PR_OpenFile^M
 * DESCRIPTION:^M
 *     Open a file for reading, writing, or both.^M
 *     PR_OpenFile has the same prototype as PR_Open but implements^M
 *     the specified file mode where possible.^M
 **************************************************************************^M
 */^M
^M
/* File mode bits */^M
#define PR_IRWXU 00700  /* read, write, execute/search by owner */^M
#define PR_IRUSR 00400  /* read permission, owner */^M
#define PR_IWUSR 00200  /* write permission, owner */^M
#define PR_IXUSR 00100  /* execute/search permission, owner */^M
#define PR_IRWXG 00070  /* read, write, execute/search by group */^M
#define PR_IRGRP 00040  /* read permission, group */^M
#define PR_IWGRP 00020  /* write permission, group */^M
#define PR_IXGRP 00010  /* execute/search permission, group */^M
#define PR_IRWXO 00007  /* read, write, execute/search by others */^M
#define PR_IROTH 00004  /* read permission, others */^M
#define PR_IWOTH 00002  /* write permission, others */^M
#define PR_IXOTH 00001  /* execute/search permission, others */^M
^M
NSPR_API(PRFileDesc*) PR_OpenFile(^M
    const char *name, PRIntn flags, PRIntn mode);^M
^M
/*^M
 **************************************************************************^M
 * FUNCTION: PR_Close^M
 * DESCRIPTION:^M
 *     Close a file or socket.^M
 * INPUTS:^M
 *     PRFileDesc *fd^M
 *         a pointer to a PRFileDesc.^M
 * OUTPUTS:^M
 *     None.^M
 * RETURN:^M
 *     PRStatus^M
 * SIDE EFFECTS:^M
 * RESTRICTIONS:^M
 *     None.^M
 * MEMORY:^M
 *     The dynamic memory pointed to by the argument fd is freed.^M
 **************************************************************************^M
 */^M
^M
NSPR_API(PRStatus)    PR_Close(PRFileDesc *fd);^M
^M
/*^M
 **************************************************************************^M
 * FUNCTION: PR_Read^M
 * DESCRIPTION:^M
 *     Read bytes from a file or socket.^M
 *     The operation will block until either an end of stream indication is^M
 *     encountered, some positive number of bytes are transferred, or there^M
 *     is an error. No more than 'amount' bytes will be transferred.^M
 * INPUTS:^M
 *     PRFileDesc *fd^M
 *         pointer to the PRFileDesc object for the file or socket^M
 *     void *buf^M
 *         pointer to a buffer to hold the data read in.^M
 *     PRInt32 amount^M
 *         the size of 'buf' (in bytes)^M
 * OUTPUTS:^M
 * RETURN:^M
 *     PRInt32^M
 *         a positive number indicates the number of bytes actually read in.^M
 *         0 means end of file is reached or the network connection is closed.^M
 *         -1 indicates a failure. The reason for the failure is obtained^M
 *         by calling PR_GetError().^M
 * SIDE EFFECTS:^M
 *     data is written into the buffer pointed to by 'buf'.^M
 * RESTRICTIONS:^M
 *     None.^M
 * MEMORY:^M
 *     N/A^M
 * ALGORITHM:^M
 *     N/A^M
 **************************************************************************^M
 */^M
^M
NSPR_API(PRInt32) PR_Read(PRFileDesc *fd, void *buf, PRInt32 amount);^M
^M
/*^M
 ***************************************************************************^M
 * FUNCTION: PR_Write^M
 * DESCRIPTION:^M
 *     Write a specified number of bytes to a file or socket.  The thread^M
 *     invoking this function blocks until all the data is written.^M
 * INPUTS:^M
 *     PRFileDesc *fd^M
 *         pointer to a PRFileDesc object that refers to a file or socket^M
 *     const void *buf^M
 *         pointer to the buffer holding the data^M
 *     PRInt32 amount^M
 *         amount of data in bytes to be written from the buffer^M
 * OUTPUTS:^M
 *     None.^M
 * RETURN: PRInt32^M
 *     A positive number indicates the number of bytes successfully written.^M
 *     A -1 is an indication that the operation failed. The reason^M
 *     for the failure is obtained by calling PR_GetError().^M
 ***************************************************************************^M
 */^M
^M
NSPR_API(PRInt32) PR_Write(PRFileDesc *fd,const void *buf,PRInt32 amount);^M
^M
/*^M
 ***************************************************************************^M
 * FUNCTION: PR_Writev^M
 * DESCRIPTION:^M
 *     Write data to a socket.  The data is organized in a PRIOVec array. The^M
 *     operation will block until all the data is written or the operation^M
 *     fails.^M
 * INPUTS:^M
 *     PRFileDesc *fd^M
 *         Pointer that points to a PRFileDesc object for a socket.^M
 *     const PRIOVec *iov^M
 *         An array of PRIOVec.  PRIOVec is a struct with the following^M
 *         two fields:^M
 *             char *iov_base;^M
 *             int iov_len;^M
 *     PRInt32 iov_size^M
 *         Number of elements in the iov array. The value of this^M
 *         argument must not be greater than PR_MAX_IOVECTOR_SIZE.^M
 *         If it is, the method will fail (PR_BUFFER_OVERFLOW_ERROR).^M
 *     PRIntervalTime timeout^M
 *       Time limit for completion of the entire write operation.^M
 * OUTPUTS:^M
 *     None^M
 * RETURN:^M
 *     A positive number indicates the number of bytes successfully written.^M
 *     A -1 is an indication that the operation failed. The reason^M
 *     for the failure is obtained by calling PR_GetError().^M
 ***************************************************************************^M
 */^M
^M
#define PR_MAX_IOVECTOR_SIZE 16   /* 'iov_size' must be <= */^M
^M
NSPR_API(PRInt32) PR_Writev(^M
    PRFileDesc *fd, const PRIOVec *iov, PRInt32 iov_size,^M
    PRIntervalTime timeout);^M
^M
/*^M
 ***************************************************************************^M
 * FUNCTION: PR_Delete^M
 * DESCRIPTION:^M
 *     Delete a file from the filesystem. The operation may fail if the^M
 *     file is open.^M
 * INPUTS:^M
 *     const char *name^M
 *         Path name of the file to be deleted.^M
 * OUTPUTS:^M
 *     None.^M
 * RETURN: PRStatus^M
 *     The function returns PR_SUCCESS if the file is successfully^M
 *     deleted, otherwise it returns PR_FAILURE.^M
 ***************************************************************************^M
 */^M
^M
NSPR_API(PRStatus) PR_Delete(const char *name);^M
^M
/**************************************************************************/^M
^M
typedef enum PRFileType^M
{^M
    PR_FILE_FILE = 1,^M
    PR_FILE_DIRECTORY = 2,^M
    PR_FILE_OTHER = 3^M
} PRFileType;^M
^M
struct PRFileInfo {^M
    PRFileType type;        /* Type of file */^M
    PROffset32 size;        /* Size, in bytes, of file's contents */^M
    PRTime creationTime;    /* Creation time per definition of PRTime */^M
    PRTime modifyTime;      /* Last modification time per definition of PRTime */^M
};^M
^M
struct PRFileInfo64 {^M
    PRFileType type;        /* Type of file */^M
    PROffset64 size;        /* Size, in bytes, of file's contents */^M
    PRTime creationTime;    /* Creation time per definition of PRTime */^M
    PRTime modifyTime;      /* Last modification time per definition of PRTime */^M
};^M
^M
/****************************************************************************^M
 * FUNCTION: PR_GetFileInfo, PR_GetFileInfo64^M
 * DESCRIPTION:^M
 *     Get the information about the file with the given path name. This is^M
 *     applicable only to NSFileDesc describing 'file' types (see^M
 * INPUTS:^M
 *     const char *fn^M
 *         path name of the file^M
 * OUTPUTS:^M
 *     PRFileInfo *info^M
 *         Information about the given file is written into the file^M
 *         information object pointer to by 'info'.^M
 * RETURN: PRStatus^M
 *     PR_GetFileInfo returns PR_SUCCESS if file information is successfully^M
 *     obtained, otherwise it returns PR_FAILURE.^M
 ***************************************************************************^M
 */^M
^M
NSPR_API(PRStatus) PR_GetFileInfo(const char *fn, PRFileInfo *info);^M
NSPR_API(PRStatus) PR_GetFileInfo64(const char *fn, PRFileInfo64 *info);^M
^M
/*^M
 **************************************************************************^M
 * FUNCTION: PR_GetOpenFileInfo, PR_GetOpenFileInfo64^M
 * DESCRIPTION:^M
 *     Get information about an open file referred to by the^M
 *     given PRFileDesc object.^M
 * INPUTS:^M
 *     const PRFileDesc *fd^M
 *          A reference to a valid, open file.^M
 * OUTPUTS:^M
 *     Same as PR_GetFileInfo, PR_GetFileInfo64^M
 * RETURN: PRStatus^M
 *     PR_GetFileInfo returns PR_SUCCESS if file information is successfully^M
 *     obtained, otherwise it returns PR_FAILURE.^M
 ***************************************************************************^M
 */^M
^M
NSPR_API(PRStatus) PR_GetOpenFileInfo(PRFileDesc *fd, PRFileInfo *info);^M
NSPR_API(PRStatus) PR_GetOpenFileInfo64(PRFileDesc *fd, PRFileInfo64 *info);^M
^M
/*^M
 **************************************************************************^M
 * FUNCTION: PR_Rename^M
 * DESCRIPTION:^M
 *     Rename a file from the old name 'from' to the new name 'to'.^M
 * INPUTS:^M
 *     const char *from^M
 *         The old name of the file to be renamed.^M
 *     const char *to^M
 *         The new name of the file.^M
 * OUTPUTS:^M
 *     None.^M
 * RETURN: PRStatus^M
 **************************************************************************^M
 */^M
^M
NSPR_API(PRStatus)    PR_Rename(const char *from, const char *to);^M
^M
/*^M
 *************************************************************************^M
 * FUNCTION: PR_Access^M
 * DESCRIPTION:^M
 *     Determine accessibility of a file.^M
 * INPUTS:^M
 *     const char *name^M
 *         path name of the file^M
 *     PRAccessHow how^M
 *         specifies which access permission to check for.^M
 *         It can be one of the following values:^M
 *             PR_ACCESS_READ_OK       Test for read permission^M
 *             PR_ACCESS_WRITE_OK      Test for write permission^M
 *             PR_ACCESS_EXISTS        Check existence of file^M
 * OUTPUTS:^M
 *     None.^M
 * RETURN: PRStatus^M
 *     PR_SUCCESS is returned if the requested access is permitted.^M
 *     Otherwise, PR_FAILURE is returned. Additional information^M
 *     regarding the reason for the failure may be retrieved from^M
 *     PR_GetError().^M
 *************************************************************************^M
 */^M
^M
typedef enum PRAccessHow {^M
    PR_ACCESS_EXISTS = 1,^M
    PR_ACCESS_WRITE_OK = 2,^M
    PR_ACCESS_READ_OK = 3^M
} PRAccessHow;^M
^M
NSPR_API(PRStatus) PR_Access(const char *name, PRAccessHow how);^M
^M
/*^M
 *************************************************************************^M
 * FUNCTION: PR_Seek, PR_Seek64^M
 * DESCRIPTION:^M
 *     Moves read-write file offset^M
 * INPUTS:^M
 *     PRFileDesc *fd^M
 *         Pointer to a PRFileDesc object.^M
 *     PROffset32, PROffset64 offset^M
 *         Specifies a value, in bytes, that is used in conjunction^M
 *         with the 'whence' parameter to set the file pointer.  A^M
 *         negative value causes seeking in the reverse direction.^M
 *     PRSeekWhence whence^M
 *         Specifies how to interpret the 'offset' parameter in setting^M
 *         the file pointer associated with the 'fd' parameter.^M
 *         Values for the 'whence' parameter are:^M
 *             PR_SEEK_SET  Sets the file pointer to the value of the^M
 *                          'offset' parameter^M
 *             PR_SEEK_CUR  Sets the file pointer to its current location^M
 *                          plus the value of the offset parameter.^M
 *             PR_SEEK_END  Sets the file pointer to the size of the^M
 *                          file plus the value of the offset parameter.^M
 * OUTPUTS:^M
 *     None.^M
 * RETURN: PROffset32, PROffset64^M
 *     Upon successful completion, the resulting pointer location,^M
 *     measured in bytes from the beginning of the file, is returned.^M
 *     If the PR_Seek() function fails, the file offset remains^M
 *     unchanged, and the returned value is -1. The error code can^M
 *     then be retrieved via PR_GetError().^M
 *************************************************************************^M
 */^M
^M
NSPR_API(PROffset32) PR_Seek(PRFileDesc *fd, PROffset32 offset, PRSeekWhence whence);^M
NSPR_API(PROffset64) PR_Seek64(PRFileDesc *fd, PROffset64 offset, PRSeekWhence whence);^M
^M
/*^M
 ************************************************************************^M
 * FUNCTION: PR_Available^M
 * DESCRIPTION:^M
 *     Determine the amount of data in bytes available for reading^M
 *     in the given file or socket.^M
 * INPUTS:^M
 *     PRFileDesc *fd^M
 *         Pointer to a PRFileDesc object that refers to a file or^M
 *         socket.^M
 * OUTPUTS:^M
 *     None^M
 * RETURN: PRInt32, PRInt64^M
 *     Upon successful completion, PR_Available returns the number of^M
 *     bytes beyond the current read pointer that is available for^M
 *     reading.  Otherwise, it returns a -1 and the reason for the^M
 *     failure can be retrieved via PR_GetError().^M
 ************************************************************************^M
 */^M
^M
NSPR_API(PRInt32) PR_Available(PRFileDesc *fd);^M
NSPR_API(PRInt64) PR_Available64(PRFileDesc *fd);^M
^M
/*^M
 ************************************************************************^M
 * FUNCTION: PR_Sync^M
 * DESCRIPTION:^M
 *     Sync any buffered data for a fd to its backing device (disk).^M
 * INPUTS:^M
 *     PRFileDesc *fd^M
 *         Pointer to a PRFileDesc object that refers to a file or^M
 *         socket^M
 * OUTPUTS:^M
 *     None^M
 * RETURN: PRStatus^M
 *     PR_SUCCESS is returned if the requested access is permitted.^M
 *     Otherwise, PR_FAILURE is returned.^M
 ************************************************************************^M
 */^M
^M
NSPR_API(PRStatus)	PR_Sync(PRFileDesc *fd);^M
^M
/************************************************************************/^M
^M
struct PRDirEntry {^M
    const char *name;        /* name of entry, relative to directory name */^M
};^M
^M
#if !defined(NO_NSPR_10_SUPPORT)^M
#define PR_DirName(dirEntry)	(dirEntry->name)^M
#endif^M
^M
/*^M
 *************************************************************************^M
 * FUNCTION: PR_OpenDir^M
 * DESCRIPTION:^M
 *     Open the directory by the given name^M
 * INPUTS:^M
 *     const char *name^M
 *         path name of the directory to be opened^M
 * OUTPUTS:^M
 *     None^M
 * RETURN: PRDir *^M
 *     If the directory is sucessfully opened, a PRDir object is^M
 *     dynamically allocated and a pointer to it is returned.^M
 *     If the directory cannot be opened, a NULL pointer is returned.^M
 * MEMORY:^M
 *     Upon successful completion, the return value points to^M
 *     dynamically allocated memory.^M
 *************************************************************************^M
 */^M
^M
NSPR_API(PRDir*) PR_OpenDir(const char *name);^M
^M
/*^M
 *************************************************************************^M
 * FUNCTION: PR_ReadDir^M
 * DESCRIPTION:^M
 * INPUTS:^M
 *     PRDir *dir^M
 *         pointer to a PRDir object that designates an open directory^M
 *     PRDirFlags flags^M
 *           PR_SKIP_NONE     Do not skip any files^M
 *           PR_SKIP_DOT      Skip the directory entry "." that^M
 *                            represents the current directory^M
 *           PR_SKIP_DOT_DOT  Skip the directory entry ".." that^M
 *                            represents the parent directory.^M
 *           PR_SKIP_BOTH     Skip both '.' and '..'^M
 *           PR_SKIP_HIDDEN   Skip hidden files^M
 * OUTPUTS:^M
 * RETURN: PRDirEntry*^M
 *     Returns a pointer to the next entry in the directory.  Returns^M
 *     a NULL pointer upon reaching the end of the directory or when an^M
 *     error occurs. The actual reason can be retrieved via PR_GetError().^M
 *************************************************************************^M
 */^M
^M
typedef enum PRDirFlags {^M
    PR_SKIP_NONE = 0x0,^M
    PR_SKIP_DOT = 0x1,^M
    PR_SKIP_DOT_DOT = 0x2,^M
    PR_SKIP_BOTH = 0x3,^M
    PR_SKIP_HIDDEN = 0x4^M
} PRDirFlags;^M
^M
NSPR_API(PRDirEntry*) PR_ReadDir(PRDir *dir, PRDirFlags flags);^M
^M
/*^M
 *************************************************************************^M
 * FUNCTION: PR_CloseDir^M
 * DESCRIPTION:^M
 *     Close the specified directory.^M
 * INPUTS:^M
 *     PRDir *dir^M
 *        The directory to be closed.^M
 * OUTPUTS:^M
 *     None^M
 * RETURN: PRStatus^M
 *        If successful, will return a status of PR_SUCCESS. Otherwise^M
 *        a value of PR_FAILURE. The reason for the failure may be re-^M
 *        trieved using PR_GetError().^M
 *************************************************************************^M
 */^M
^M
NSPR_API(PRStatus) PR_CloseDir(PRDir *dir);^M
^M
/*^M
 *************************************************************************^M
 * FUNCTION: PR_MkDir^M
 * DESCRIPTION:^M
 *     Create a new directory with the given name and access mode.^M
 * INPUTS:^M
 *     const char *name^M
 *        The name of the directory to be created. All the path components^M
 *        up to but not including the leaf component must already exist.^M
 *     PRIntn mode^M
 *        See 'mode' definiton in PR_Open().^M
 * OUTPUTS:^M
 *     None^M
 * RETURN: PRStatus^M
 *        If successful, will return a status of PR_SUCCESS. Otherwise^M
 *        a value of PR_FAILURE. The reason for the failure may be re-^M
 *        trieved using PR_GetError().^M
 *************************************************************************^M
 */^M
^M
NSPR_API(PRStatus) PR_MkDir(const char *name, PRIntn mode);^M
^M
/*^M
 *************************************************************************^M
 * FUNCTION: PR_MakeDir^M
 * DESCRIPTION:^M
 *     Create a new directory with the given name and access mode.^M
 *     PR_MakeDir has the same prototype as PR_MkDir but implements^M
 *     the specified access mode where possible.^M
 *************************************************************************^M
 */^M
^M
NSPR_API(PRStatus) PR_MakeDir(const char *name, PRIntn mode);^M
^M
/*^M
 *************************************************************************^M
 * FUNCTION: PR_RmDir^M
 * DESCRIPTION:^M
 *     Remove a directory by the given name.^M
 * INPUTS:^M
 *     const char *name^M
 *        The name of the directory to be removed. All the path components^M
 *        must already exist. Only the leaf component will be removed.^M
 * OUTPUTS:^M
 *     None^M
 * RETURN: PRStatus^M
 *        If successful, will return a status of PR_SUCCESS. Otherwise^M
 *        a value of PR_FAILURE. The reason for the failure may be re-^M
 *        trieved using PR_GetError().^M
 **************************************************************************^M
 */^M
^M
NSPR_API(PRStatus) PR_RmDir(const char *name);^M
^M
/*^M
 *************************************************************************^M
 * FUNCTION: PR_NewUDPSocket^M
 * DESCRIPTION:^M
 *     Create a new UDP socket.^M
 * INPUTS:^M
 *     None^M
 * OUTPUTS:^M
 *     None^M
 * RETURN: PRFileDesc*^M
 *     Upon successful completion, PR_NewUDPSocket returns a pointer^M
 *     to the PRFileDesc created for the newly opened UDP socket.^M
 *     Returns a NULL pointer if the creation of a new UDP socket failed.^M
 *^M
 **************************************************************************^M
 */^M
^M
NSPR_API(PRFileDesc*)    PR_NewUDPSocket(void);^M
^M
/*^M
 *************************************************************************^M
 * FUNCTION: PR_NewTCPSocket^M
 * DESCRIPTION:^M
 *     Create a new TCP socket.^M
 * INPUTS:^M
 *     None^M
 * OUTPUTS:^M
 *     None^M
 * RETURN: PRFileDesc*^M
 *     Upon successful completion, PR_NewTCPSocket returns a pointer^M
 *     to the PRFileDesc created for the newly opened TCP socket.^M
 *     Returns a NULL pointer if the creation of a new TCP socket failed.^M
 *^M
 **************************************************************************^M
 */^M
^M
NSPR_API(PRFileDesc*)    PR_NewTCPSocket(void);^M
^M
/*^M
 *************************************************************************^M
 * FUNCTION: PR_OpenUDPSocket^M
 * DESCRIPTION:^M
 *     Create a new UDP socket of the specified address family.^M
 * INPUTS:^M
 *     PRIntn af^M
 *       Address family^M
 * OUTPUTS:^M
 *     None^M
 * RETURN: PRFileDesc*^M
 *     Upon successful completion, PR_OpenUDPSocket returns a pointer^M
 *     to the PRFileDesc created for the newly opened UDP socket.^M
 *     Returns a NULL pointer if the creation of a new UDP socket failed.^M
 *^M
 **************************************************************************^M
 */^M
^M
NSPR_API(PRFileDesc*)    PR_OpenUDPSocket(PRIntn af);^M
^M
/*^M
 *************************************************************************^M
 * FUNCTION: PR_OpenTCPSocket^M
 * DESCRIPTION:^M
 *     Create a new TCP socket of the specified address family.^M
 * INPUTS:^M
 *     PRIntn af^M
 *       Address family^M
 * OUTPUTS:^M
 *     None^M
 * RETURN: PRFileDesc*^M
 *     Upon successful completion, PR_NewTCPSocket returns a pointer^M
 *     to the PRFileDesc created for the newly opened TCP socket.^M
 *     Returns a NULL pointer if the creation of a new TCP socket failed.^M
 *^M
 **************************************************************************^M
 */^M
^M
NSPR_API(PRFileDesc*)    PR_OpenTCPSocket(PRIntn af);^M
^M
/*^M
 *************************************************************************^M
 * FUNCTION: PR_Connect^M
 * DESCRIPTION:^M
 *     Initiate a connection on a socket.^M
 * INPUTS:^M
 *     PRFileDesc *fd^M
 *       Points to a PRFileDesc object representing a socket^M
 *     PRNetAddr *addr^M
 *       Specifies the address of the socket in its own communication^M
 *       space.^M
 *     PRIntervalTime timeout^M
 *       Time limit for completion of the connect operation.^M
 * OUTPUTS:^M
 *     None^M
 * RETURN: PRStatus^M
 *     Upon successful completion of connection initiation, PR_Connect^M
 *     returns PR_SUCCESS.  Otherwise, it returns PR_FAILURE.  Further^M
 *     failure information can be obtained by calling PR_GetError().^M
 **************************************************************************^M
 */^M
^M
NSPR_API(PRStatus) PR_Connect(^M
    PRFileDesc *fd, const PRNetAddr *addr, PRIntervalTime timeout);^M
^M
/*^M
 *************************************************************************^M
 * FUNCTION: PR_ConnectContinue^M
 * DESCRIPTION:^M
 *     Continue a nonblocking connect.  After a nonblocking connect^M
 *     is initiated with PR_Connect() (which fails with^M
 *     PR_IN_PROGRESS_ERROR), one should call PR_Poll() on the socket,^M
 *     with the in_flags PR_POLL_WRITE | PR_POLL_EXCEPT.  When^M
 *     PR_Poll() returns, one calls PR_ConnectContinue() on the^M
 *     socket to determine whether the nonblocking connect has^M
 *     completed or is still in progress.  Repeat the PR_Poll(),^M
 *     PR_ConnectContinue() sequence until the nonblocking connect^M
 *     has completed.^M
 * INPUTS:^M
 *     PRFileDesc *fd^M
 *         the file descriptor representing a socket^M
 *     PRInt16 out_flags^M
 *         the out_flags field of the poll descriptor returned by^M
 *         PR_Poll()^M
 * RETURN: PRStatus^M
 *     If the nonblocking connect has successfully completed,^M
 *     PR_GetConnectStatus returns PR_SUCCESS.  If PR_GetConnectStatus()^M
 *     returns PR_FAILURE, call PR_GetError():^M
 *     - PR_IN_PROGRESS_ERROR: the nonblocking connect is still in^M
 *       progress and has not completed yet.  The caller should poll^M
 *       on the file descriptor for the in_flags^M
 *       PR_POLL_WRITE|PR_POLL_EXCEPT and retry PR_ConnectContinue^M
 *       later when PR_Poll() returns.^M
 *     - Other errors: the nonblocking connect has failed with this^M
 *       error code.^M
 */^M
^M
NSPR_API(PRStatus)    PR_ConnectContinue(PRFileDesc *fd, PRInt16 out_flags);^M
^M
/*^M
 *************************************************************************^M
 * THIS FUNCTION IS DEPRECATED.  USE PR_ConnectContinue INSTEAD.^M
 *^M
 * FUNCTION: PR_GetConnectStatus^M
 * DESCRIPTION:^M
 *     Get the completion status of a nonblocking connect.  After^M
 *     a nonblocking connect is initiated with PR_Connect() (which^M
 *     fails with PR_IN_PROGRESS_ERROR), one should call PR_Poll()^M
 *     on the socket, with the in_flags PR_POLL_WRITE | PR_POLL_EXCEPT.^M
 *     When PR_Poll() returns, one calls PR_GetConnectStatus on the^M
 *     PRPollDesc structure to determine whether the nonblocking^M
 *     connect has succeeded or failed.^M
 * INPUTS:^M
 *     const PRPollDesc *pd^M
 *         Pointer to a PRPollDesc whose fd member is the socket,^M
 *         and in_flags must contain PR_POLL_WRITE and PR_POLL_EXCEPT.^M
 *         PR_Poll() should have been called and set the out_flags.^M
 * RETURN: PRStatus^M
 *     If the nonblocking connect has successfully completed,^M
 *     PR_GetConnectStatus returns PR_SUCCESS.  If PR_GetConnectStatus()^M
 *     returns PR_FAILURE, call PR_GetError():^M
 *     - PR_IN_PROGRESS_ERROR: the nonblocking connect is still in^M
 *       progress and has not completed yet.^M
 *     - Other errors: the nonblocking connect has failed with this^M
 *       error code.^M
 */^M
^M
NSPR_API(PRStatus)    PR_GetConnectStatus(const PRPollDesc *pd);^M
^M
/*^M
 *************************************************************************^M
 * FUNCTION: PR_Accept^M
 * DESCRIPTION:^M
 *     Accept a connection on a socket.^M
 * INPUTS:^M
 *     PRFileDesc *fd^M
 *       Points to a PRFileDesc object representing the rendezvous socket^M
 *       on which the caller is willing to accept new connections.^M
 *     PRIntervalTime timeout^M
 *       Time limit for completion of the accept operation.^M
 * OUTPUTS:^M
 *     PRNetAddr *addr^M
 *       Returns the address of the connecting entity in its own^M
 *       communication space. It may be NULL.^M
 * RETURN: PRFileDesc*^M
 *     Upon successful acceptance of a connection, PR_Accept^M
 *     returns a valid file descriptor. Otherwise, it returns NULL.^M
 *     Further failure information can be obtained by calling PR_GetError().^M
 **************************************************************************^M
 */^M
^M
NSPR_API(PRFileDesc*) PR_Accept(^M
    PRFileDesc *fd, PRNetAddr *addr, PRIntervalTime timeout);^M
^M
/*^M
 *************************************************************************^M
 * FUNCTION: PR_Bind^M
 * DESCRIPTION:^M
 *    Bind an address to a socket.^M
 * INPUTS:^M
 *     PRFileDesc *fd^M
 *       Points to a PRFileDesc object representing a socket.^M
 *     PRNetAddr *addr^M
 *       Specifies the address to which the socket will be bound.^M
 * OUTPUTS:^M
 *     None^M
 * RETURN: PRStatus^M
 *     Upon successful binding of an address to a socket, PR_Bind^M
 *     returns PR_SUCCESS.  Otherwise, it returns PR_FAILURE.  Further^M
 *     failure information can be obtained by calling PR_GetError().^M
 **************************************************************************^M
 */^M
^M
NSPR_API(PRStatus) PR_Bind(PRFileDesc *fd, const PRNetAddr *addr);^M
^M
/*^M
 *************************************************************************^M
 * FUNCTION: PR_Listen^M
 * DESCRIPTION:^M
 *    Listen for connections on a socket.^M
 * INPUTS:^M
 *     PRFileDesc *fd^M
 *       Points to a PRFileDesc object representing a socket that will be^M
 *       used to listen for new connections.^M
 *     PRIntn backlog^M
 *       Specifies the maximum length of the queue of pending connections.^M
 * OUTPUTS:^M
 *     None^M
 * RETURN: PRStatus^M
 *     Upon successful completion of listen request, PR_Listen^M
 *     returns PR_SUCCESS.  Otherwise, it returns PR_FAILURE.  Further^M
 *     failure information can be obtained by calling PR_GetError().^M
 **************************************************************************^M
 */^M
^M
NSPR_API(PRStatus) PR_Listen(PRFileDesc *fd, PRIntn backlog);^M
^M
/*^M
 *************************************************************************^M
 * FUNCTION: PR_Shutdown^M
 * DESCRIPTION:^M
 *    Shut down part of a full-duplex connection on a socket.^M
 * INPUTS:^M
 *     PRFileDesc *fd^M
 *       Points to a PRFileDesc object representing a connected socket.^M
 *     PRIntn how^M
 *       Specifies the kind of disallowed operations on the socket.^M
 *           PR_SHUTDOWN_RCV - Further receives will be disallowed^M
 *           PR_SHUTDOWN_SEND - Further sends will be disallowed^M
 *           PR_SHUTDOWN_BOTH - Further sends and receives will be disallowed^M
 * OUTPUTS:^M
 *     None^M
 * RETURN: PRStatus^M
 *     Upon successful completion of shutdown request, PR_Shutdown^M
 *     returns PR_SUCCESS.  Otherwise, it returns PR_FAILURE.  Further^M
 *     failure information can be obtained by calling PR_GetError().^M
 **************************************************************************^M
 */^M
^M
typedef enum PRShutdownHow^M
{^M
    PR_SHUTDOWN_RCV = 0,      /* disallow further receives */^M
    PR_SHUTDOWN_SEND = 1,     /* disallow further sends */^M
    PR_SHUTDOWN_BOTH = 2      /* disallow further receives and sends */^M
} PRShutdownHow;^M
^M
NSPR_API(PRStatus)    PR_Shutdown(PRFileDesc *fd, PRShutdownHow how);^M
^M
/*^M
 *************************************************************************^M
 * FUNCTION: PR_Recv^M
 * DESCRIPTION:^M
 *    Receive a specified number of bytes from a connected socket.^M
 *     The operation will block until some positive number of bytes are ^M
 *     transferred, a time out has occurred, or there is an error. ^M
 *     No more than 'amount' bytes will be transferred.^M
 * INPUTS:^M
 *     PRFileDesc *fd^M
 *       points to a PRFileDesc object representing a socket.^M
 *     void *buf^M
 *       pointer to a buffer to hold the data received.^M
 *     PRInt32 amount^M
 *       the size of 'buf' (in bytes)^M
 *     PRIntn flags^M
 *       must be zero or PR_MSG_PEEK.^M
 *     PRIntervalTime timeout^M
 *       Time limit for completion of the receive operation.^M
 * OUTPUTS:^M
 *     None^M
 * RETURN: PRInt32^M
 *         a positive number indicates the number of bytes actually received.^M
 *         0 means the network connection is closed.^M
 *         -1 indicates a failure. The reason for the failure is obtained^M
 *         by calling PR_GetError().^M
 **************************************************************************^M
 */^M
^M
#define PR_MSG_PEEK 0x2^M
^M
NSPR_API(PRInt32)    PR_Recv(PRFileDesc *fd, void *buf, PRInt32 amount,^M
                PRIntn flags, PRIntervalTime timeout);^M
^M
/*^M
 *************************************************************************^M
 * FUNCTION: PR_Send^M
 * DESCRIPTION:^M
 *    Send a specified number of bytes from a connected socket.^M
 *     The operation will block until all bytes are ^M
 *     processed, a time out has occurred, or there is an error. ^M
 * INPUTS:^M
 *     PRFileDesc *fd^M
 *       points to a PRFileDesc object representing a socket.^M
 *     void *buf^M
 *       pointer to a buffer from where the data is sent.^M
 *     PRInt32 amount^M
 *       the size of 'buf' (in bytes)^M
 *     PRIntn flags^M
 *        (OBSOLETE - must always be zero)^M
 *     PRIntervalTime timeout^M
 *       Time limit for completion of the send operation.^M
 * OUTPUTS:^M
 *     None^M
 * RETURN: PRInt32^M
 *     A positive number indicates the number of bytes successfully processed.^M
 *     This number must always equal 'amount'. A -1 is an indication that the^M
 *     operation failed. The reason for the failure is obtained by calling^M
 *     PR_GetError().^M
 **************************************************************************^M
 */^M
^M
NSPR_API(PRInt32)    PR_Send(PRFileDesc *fd, const void *buf, PRInt32 amount,^M
                                PRIntn flags, PRIntervalTime timeout);^M
^M
/*^M
 *************************************************************************^M
 * FUNCTION: PR_RecvFrom^M
 * DESCRIPTION:^M
 *     Receive up to a specified number of bytes from socket which may^M
 *     or may not be connected.^M
 *     The operation will block until one or more bytes are ^M
 *     transferred, a time out has occurred, or there is an error. ^M
 *     No more than 'amount' bytes will be transferred.^M
 * INPUTS:^M
 *     PRFileDesc *fd^M
 *       points to a PRFileDesc object representing a socket.^M
 *     void *buf^M
 *       pointer to a buffer to hold the data received.^M
 *     PRInt32 amount^M
 *       the size of 'buf' (in bytes)^M
 *     PRIntn flags^M
 *        (OBSOLETE - must always be zero)^M
 *     PRNetAddr *addr^M
 *       Specifies the address of the sending peer. It may be NULL.^M
 *     PRIntervalTime timeout^M
 *       Time limit for completion of the receive operation.^M
 * OUTPUTS:^M
 *     None^M
 * RETURN: PRInt32^M
 *         a positive number indicates the number of bytes actually received.^M
 *         0 means the network connection is closed.^M
 *         -1 indicates a failure. The reason for the failure is obtained^M
 *         by calling PR_GetError().^M
 **************************************************************************^M
 */^M
^M
NSPR_API(PRInt32) PR_RecvFrom(^M
    PRFileDesc *fd, void *buf, PRInt32 amount, PRIntn flags,^M
    PRNetAddr *addr, PRIntervalTime timeout);^M
^M
/*^M
 *************************************************************************^M
 * FUNCTION: PR_SendTo^M
 * DESCRIPTION:^M
 *    Send a specified number of bytes from an unconnected socket.^M
 *    The operation will block until all bytes are ^M
 *    sent, a time out has occurred, or there is an error. ^M
 * INPUTS:^M
 *     PRFileDesc *fd^M
 *       points to a PRFileDesc object representing an unconnected socket.^M
 *     void *buf^M
 *       pointer to a buffer from where the data is sent.^M
 *     PRInt32 amount^M
 *       the size of 'buf' (in bytes)^M
 *     PRIntn flags^M
 *        (OBSOLETE - must always be zero)^M
 *     PRNetAddr *addr^M
 *       Specifies the address of the peer.^M
.*     PRIntervalTime timeout^M
 *       Time limit for completion of the send operation.^M
 * OUTPUTS:^M
 *     None^M
 * RETURN: PRInt32^M
 *     A positive number indicates the number of bytes successfully sent.^M
 *     -1 indicates a failure. The reason for the failure is obtained^M
 *     by calling PR_GetError().^M
 **************************************************************************^M
 */^M
^M
NSPR_API(PRInt32) PR_SendTo(^M
    PRFileDesc *fd, const void *buf, PRInt32 amount, PRIntn flags,^M
    const PRNetAddr *addr, PRIntervalTime timeout);^M
^M
/*^M
*************************************************************************^M
** FUNCTION: PR_TransmitFile^M
** DESCRIPTION:^M
**    Transmitfile sends a complete file (sourceFile) across a socket ^M
**    (networkSocket).  If headers is non-NULL, the headers will be sent across^M
**    the socket prior to sending the file.^M
** ^M
**    Optionally, the PR_TRANSMITFILE_CLOSE_SOCKET flag may be passed to^M
**    transmitfile.  This flag specifies that transmitfile should close the^M
**    socket after sending the data.^M
**^M
** INPUTS:^M
**    PRFileDesc *networkSocket^M
**        The socket to send data over^M
**    PRFileDesc *sourceFile^M
**        The file to send^M
**    const void *headers^M
**        A pointer to headers to be sent before sending data^M
**    PRInt32       hlen^M
**        length of header buffers in bytes.^M
**    PRTransmitFileFlags       flags^M
**        If the flags indicate that the connection should be closed,^M
**        it will be done immediately after transferring the file, unless^M
**        the operation is unsuccessful. ^M
.*     PRIntervalTime timeout^M
 *        Time limit for completion of the transmit operation.^M
**^M
** RETURNS:^M
**    Returns the number of bytes written or -1 if the operation failed.^M
**    If an error occurs while sending the file, the PR_TRANSMITFILE_CLOSE_^M
**    SOCKET flag is ignored. The reason for the failure is obtained^M
**    by calling PR_GetError().^M
**************************************************************************^M
*/^M
^M
NSPR_API(PRInt32) PR_TransmitFile(^M
    PRFileDesc *networkSocket, PRFileDesc *sourceFile,^M
    const void *headers, PRInt32 hlen, PRTransmitFileFlags flags,^M
    PRIntervalTime timeout);^M
^M
/*^M
*************************************************************************^M
** FUNCTION: PR_SendFile^M
** DESCRIPTION:^M
**    PR_SendFile sends data from a file (sendData->fd) across a socket ^M
**    (networkSocket).  If specified, a header and/or trailer buffer are sent^M
**	  before and after the file, respectively. The file offset, number of bytes^M
** 	  of file data to send, the header and trailer buffers are specified in the^M
**	  sendData argument.^M
** ^M
**    Optionally, if the PR_TRANSMITFILE_CLOSE_SOCKET flag is passed, the^M
**    socket is closed after successfully sending the data.^M
**^M
** INPUTS:^M
**    PRFileDesc *networkSocket^M
**        The socket to send data over^M
**    PRSendFileData *sendData^M
**        Contains the FD, file offset and length, header and trailer^M
**		  buffer specifications.^M
**    PRTransmitFileFlags       flags^M
**        If the flags indicate that the connection should be closed,^M
**        it will be done immediately after transferring the file, unless^M
**        the operation is unsuccessful. ^M
.*     PRIntervalTime timeout^M
 *        Time limit for completion of the send operation.^M
**^M
** RETURNS:^M
**    Returns the number of bytes written or -1 if the operation failed.^M
**    If an error occurs while sending the file, the PR_TRANSMITFILE_CLOSE_^M
**    SOCKET flag is ignored. The reason for the failure is obtained^M
**    by calling PR_GetError().^M
**************************************************************************^M
*/^M
^M
struct PRSendFileData {^M
	PRFileDesc	*fd;			/* file to send							*/^M
	PRUint32	file_offset;	/* file offset							*/^M
	PRSize		file_nbytes;	/* number of bytes of file data to send	*/^M
								/* if 0, send data from file_offset to	*/^M
								/* end-of-file.							*/^M
	const void	*header;		/* header buffer						*/^M
	PRInt32		hlen;			/* header len							*/^M
	const void	*trailer;		/* trailer buffer						*/^M
	PRInt32		tlen;			/* trailer len							*/^M
};^M
^M
^M
NSPR_API(PRInt32) PR_SendFile(^M
    PRFileDesc *networkSocket, PRSendFileData *sendData,^M
	PRTransmitFileFlags flags, PRIntervalTime timeout);^M
^M
/*^M
*************************************************************************^M
** FUNCTION: PR_AcceptRead^M
** DESCRIPTION:^M
**    AcceptRead accepts a new connection, returns the newly created^M
**    socket's descriptor and also returns the connecting peer's address.^M
**    AcceptRead, as its name suggests, also receives the first block of data ^M
**    sent by the peer.^M
**^M
** INPUTS:^M
**    PRFileDesc *listenSock^M
**        A socket descriptor that has been called with the PR_Listen() ^M
**        function, also known as the rendezvous socket.^M
**    void *buf^M
**        A pointer to a buffer to receive data sent by the client.  This ^M
**        buffer must be large enough to receive <amount> bytes of data^M
**        and two PRNetAddr structures, plus an extra 32 bytes. See:^M
**        PR_ACCEPT_READ_BUF_OVERHEAD.^M
**    PRInt32 amount^M
**        The number of bytes of client data to receive.  Does not include^M
**        the size of the PRNetAddr structures.  If 0, no data will be read^M
**        from the client.^M
**    PRIntervalTime timeout^M
**        The timeout interval only applies to the read portion of the ^M
**        operation.  PR_AcceptRead will block indefinitely until the ^M
**        connection is accepted; the read will timeout after the timeout ^M
**        interval elapses.^M
** OUTPUTS:^M
**    PRFileDesc **acceptedSock^M
**        The file descriptor for the newly connected socket.  This parameter^M
**        will only be valid if the function return does not indicate failure.^M
**    PRNetAddr  **peerAddr,^M
**        The address of the remote socket.  This parameter will only be^M
**        valid if the function return does not indicate failure.  The^M
**        returned address is not guaranteed to be properly aligned.^M
** ^M
** RETURNS:^M
**     The number of bytes read from the client or -1 on failure.  The reason ^M
**     for the failure is obtained by calling PR_GetError().^M
**************************************************************************^M
**/       ^M
/* define buffer overhead constant. Add this value to the user's ^M
** data length when allocating a buffer to accept data.^M
**    Example:^M
**    #define USER_DATA_SIZE 10^M
**    char buf[USER_DATA_SIZE + PR_ACCEPT_READ_BUF_OVERHEAD];^M
**    bytesRead = PR_AcceptRead( s, fd, &a, &p, USER_DATA_SIZE, ...);^M
*/^M
#define PR_ACCEPT_READ_BUF_OVERHEAD (32+(2*sizeof(PRNetAddr)))^M
^M
NSPR_API(PRInt32) PR_AcceptRead(^M
    PRFileDesc *listenSock, PRFileDesc **acceptedSock,^M
    PRNetAddr **peerAddr, void *buf, PRInt32 amount, PRIntervalTime timeout);^M
^M
/*^M
*************************************************************************^M
** FUNCTION: PR_NewTCPSocketPair^M
** DESCRIPTION:^M
**    Create a new TCP socket pair. The returned descriptors can be used^M
**    interchangeably; they are interconnected full-duplex descriptors: data^M
**    written to one can be read from the other and vice-versa.^M
**^M
** INPUTS:^M
**    None^M
** OUTPUTS:^M
**    PRFileDesc *fds[2]^M
**        The file descriptor pair for the newly created TCP sockets.^M
** RETURN: PRStatus^M
**     Upon successful completion of TCP socket pair, PR_NewTCPSocketPair ^M
**     returns PR_SUCCESS.  Otherwise, it returns PR_FAILURE.  Further^M
**     failure information can be obtained by calling PR_GetError().^M
** XXX can we implement this on windoze and mac?^M
**************************************************************************^M
**/^M
NSPR_API(PRStatus) PR_NewTCPSocketPair(PRFileDesc *fds[2]);^M
^M
/*^M
*************************************************************************^M
** FUNCTION: PR_GetSockName^M
** DESCRIPTION:^M
**    Get socket name.  Return the network address for this socket.^M
**^M
** INPUTS:^M
**     PRFileDesc *fd^M
**       Points to a PRFileDesc object representing the socket.^M
** OUTPUTS:^M
**     PRNetAddr *addr^M
**       Returns the address of the socket in its own communication space.^M
** RETURN: PRStatus^M
**     Upon successful completion, PR_GetSockName returns PR_SUCCESS.  ^M
**     Otherwise, it returns PR_FAILURE.  Further failure information can ^M
**     be obtained by calling PR_GetError().^M
**************************************************************************^M
**/^M
NSPR_API(PRStatus)	PR_GetSockName(PRFileDesc *fd, PRNetAddr *addr);^M
^M
/*^M
*************************************************************************^M
** FUNCTION: PR_GetPeerName^M
** DESCRIPTION:^M
**    Get name of the connected peer.  Return the network address for the ^M
**    connected peer socket.^M
**^M
** INPUTS:^M
**     PRFileDesc *fd^M
**       Points to a PRFileDesc object representing the connected peer.^M
** OUTPUTS:^M
**     PRNetAddr *addr^M
**       Returns the address of the connected peer in its own communication^M
**       space.^M
** RETURN: PRStatus^M
**     Upon successful completion, PR_GetPeerName returns PR_SUCCESS.  ^M
**     Otherwise, it returns PR_FAILURE.  Further failure information can ^M
**     be obtained by calling PR_GetError().^M
**************************************************************************^M
**/^M
NSPR_API(PRStatus)	PR_GetPeerName(PRFileDesc *fd, PRNetAddr *addr);^M
^M
NSPR_API(PRStatus)	PR_GetSocketOption(^M
    PRFileDesc *fd, PRSocketOptionData *data);^M
^M
NSPR_API(PRStatus)	PR_SetSocketOption(^M
    PRFileDesc *fd, const PRSocketOptionData *data);^M
^M
/*^M
 *********************************************************************^M
 *^M
 * File descriptor inheritance^M
 *^M
 *********************************************************************^M
 */^M
^M
/*^M
 ************************************************************************^M
 * FUNCTION: PR_SetFDInheritable^M
 * DESCRIPTION:^M
 *    Set the inheritance attribute of a file descriptor.^M
 *^M
 * INPUTS:^M
 *     PRFileDesc *fd^M
 *       Points to a PRFileDesc object.^M
 *     PRBool inheritable^M
 *       If PR_TRUE, the file descriptor fd is set to be inheritable^M
 *       by a child process.  If PR_FALSE, the file descriptor is set^M
 *       to be not inheritable by a child process.^M
 * RETURN: PRStatus^M
 *     Upon successful completion, PR_SetFDInheritable returns PR_SUCCESS.  ^M
 *     Otherwise, it returns PR_FAILURE.  Further failure information can ^M
 *     be obtained by calling PR_GetError().^M
 *************************************************************************^M
 */^M
NSPR_API(PRStatus) PR_SetFDInheritable(^M
    PRFileDesc *fd,^M
    PRBool inheritable);^M
^M
/*^M
 ************************************************************************^M
 * FUNCTION: PR_GetInheritedFD^M
 * DESCRIPTION:^M
 *    Get an inherited file descriptor with the specified name.^M
 *^M
 * INPUTS:^M
 *     const char *name^M
 *       The name of the inherited file descriptor.^M
 * RETURN: PRFileDesc *^M
 *     Upon successful completion, PR_GetInheritedFD returns the^M
 *     inherited file descriptor with the specified name.  Otherwise,  ^M
 *     it returns NULL.  Further failure information can be obtained^M
 *     by calling PR_GetError().^M
 *************************************************************************^M
 */^M
NSPR_API(PRFileDesc *) PR_GetInheritedFD(const char *name);^M
^M
/*^M
 *********************************************************************^M
 *^M
 * Memory-mapped files^M
 *^M
 *********************************************************************^M
 */^M
^M
typedef struct PRFileMap PRFileMap;^M
^M
/*^M
 * protection options for read and write accesses of a file mapping^M
 */^M
typedef enum PRFileMapProtect {^M
    PR_PROT_READONLY,     /* read only */^M
    PR_PROT_READWRITE,    /* readable, and write is shared */^M
    PR_PROT_WRITECOPY     /* readable, and write is private (copy-on-write) */^M
} PRFileMapProtect;^M
^M
NSPR_API(PRFileMap *) PR_CreateFileMap(^M
    PRFileDesc *fd,^M
    PRInt64 size,^M
    PRFileMapProtect prot);^M
^M
/*^M
 * return the alignment (in bytes) of the offset argument to PR_MemMap^M
 */^M
NSPR_API(PRInt32) PR_GetMemMapAlignment(void);^M
^M
NSPR_API(void *) PR_MemMap(^M
    PRFileMap *fmap,^M
    PROffset64 offset,  /* must be aligned and sized according to the^M
                         * return value of PR_GetMemMapAlignment() */^M
    PRUint32 len);^M
^M
NSPR_API(PRStatus) PR_MemUnmap(void *addr, PRUint32 len);^M
^M
NSPR_API(PRStatus) PR_CloseFileMap(PRFileMap *fmap);^M
^M
/*^M
 ******************************************************************^M
 *^M
 * Interprocess communication^M
 *^M
 ******************************************************************^M
 */^M
^M
/*^M
 * Creates an anonymous pipe and returns file descriptors for the^M
 * read and write ends of the pipe.^M
 */^M
^M
NSPR_API(PRStatus) PR_CreatePipe(^M
    PRFileDesc **readPipe,^M
    PRFileDesc **writePipe^M
);^M
^M
/************************************************************************/^M
/************** The following definitions are for poll ******************/^M
/************************************************************************/^M
^M
struct PRPollDesc {^M
    PRFileDesc* fd;^M
    PRInt16 in_flags;^M
    PRInt16 out_flags;^M
};^M
^M
/*^M
** Bit values for PRPollDesc.in_flags or PRPollDesc.out_flags. Binary-or^M
** these together to produce the desired poll request.^M
*/^M
^M
#if defined(_PR_POLL_BACKCOMPAT)^M
^M
#include <poll.h>^M
#define PR_POLL_READ    POLLIN^M
#define PR_POLL_WRITE   POLLOUT^M
#define PR_POLL_EXCEPT  POLLPRI^M
#define PR_POLL_ERR     POLLERR     /* only in out_flags */^M
#define PR_POLL_NVAL    POLLNVAL    /* only in out_flags when fd is bad */^M
#define PR_POLL_HUP     POLLHUP     /* only in out_flags */^M
^M
#else  /* _PR_POLL_BACKCOMPAT */^M
^M
#define PR_POLL_READ    0x1^M
#define PR_POLL_WRITE   0x2^M
#define PR_POLL_EXCEPT  0x4^M
#define PR_POLL_ERR     0x8         /* only in out_flags */^M
#define PR_POLL_NVAL    0x10        /* only in out_flags when fd is bad */^M
#define PR_POLL_HUP     0x20        /* only in out_flags */^M
^M
#endif  /* _PR_POLL_BACKCOMPAT */^M
^M
/*^M
*************************************************************************^M
** FUNCTION:    PR_Poll^M
** DESCRIPTION:^M
**^M
** The call returns as soon as I/O is ready on one or more of the underlying^M
** file/socket objects. A count of the number of ready descriptors is^M
** returned unless a timeout occurs in which case zero is returned.^M
**^M
** PRPollDesc.in_flags should be set to the desired request^M
** (read/write/except or some combination). Upon successful return from^M
** this call PRPollDesc.out_flags will be set to indicate what kind of^M
** i/o can be performed on the respective descriptor. PR_Poll() uses the^M
** out_flags fields as scratch variables during the call. If PR_Poll()^M
** returns 0 or -1, the out_flags fields do not contain meaningful values^M
** and must not be used.^M
**^M
** INPUTS:^M
**      PRPollDesc *pds         A pointer to an array of PRPollDesc^M
**^M
**      PRIntn npds             The number of elements in the array^M
**                              If this argument is zero PR_Poll is^M
**                              equivalent to a PR_Sleep(timeout).^M
**^M
**      PRIntervalTime timeout  Amount of time the call will block waiting^M
**                              for I/O to become ready. If this time expires^M
**                              w/o any I/O becoming ready, the result will^M
**                              be zero.^M
**^M
** OUTPUTS:    None^M
** RETURN:^M
**      PRInt32                 Number of PRPollDesc's with events or zero^M
**                              if the function timed out or -1 on failure.^M
**                              The reason for the failure is obtained by^M
**                              calling PR_GetError().^M
**************************************************************************^M
*/^M
NSPR_API(PRInt32) PR_Poll(^M
    PRPollDesc *pds, PRIntn npds, PRIntervalTime timeout);^M
^M
/*^M
**************************************************************************^M
**^M
** Pollable events^M
**^M
** A pollable event is a special kind of file descriptor.^M
** The only I/O operation you can perform on a pollable event^M
** is to poll it with the PR_POLL_READ flag.  You can't^M
** read from or write to a pollable event.^M
**^M
** The purpose of a pollable event is to combine event waiting^M
** with I/O waiting in a single PR_Poll call.  Pollable events^M
** are implemented using a pipe or a pair of TCP sockets^M
** connected via the loopback address, therefore setting and^M
** waiting for pollable events are expensive operating system^M
** calls.  Do not use pollable events for general thread^M
** synchronization. Use condition variables instead.^M
**^M
** A pollable event has two states: set and unset.  Events^M
** are not queued, so there is no notion of an event count.^M
** A pollable event is either set or unset.^M
**^M
** A new pollable event is created by a PR_NewPollableEvent^M
** call and is initially in the unset state.^M
**^M
** PR_WaitForPollableEvent blocks the calling thread until^M
** the pollable event is set, and then it atomically unsets^M
** the pollable event before it returns.^M
**^M
** To set a pollable event, call PR_SetPollableEvent.^M
**^M
** One can call PR_Poll with the PR_POLL_READ flag on a pollable^M
** event.  When the pollable event is set, PR_Poll returns with^M
** the PR_POLL_READ flag set in the out_flags.^M
**^M
** To close a pollable event, call PR_DestroyPollableEvent^M
** (not PR_Close).^M
**^M
**************************************************************************^M
*/^M
^M
NSPR_API(PRFileDesc *) PR_NewPollableEvent(void);^M
^M
NSPR_API(PRStatus) PR_DestroyPollableEvent(PRFileDesc *event);^M
^M
NSPR_API(PRStatus) PR_SetPollableEvent(PRFileDesc *event);^M
^M
NSPR_API(PRStatus) PR_WaitForPollableEvent(PRFileDesc *event);^M
^M
PR_END_EXTERN_C^M
^M
#endif /* prio_h___ */^M
