/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1998-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions required by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
/*^M
** File:                prlong.h^M
** Description: Portable access to 64 bit numerics^M
**^M
** Long-long (64-bit signed integer type) support. Some C compilers^M
** don't support 64 bit integers yet, so we use these macros to^M
** support both machines that do and don't.^M
**/^M
#ifndef prlong_h___^M
#define prlong_h___^M
^M
#include "prtypes.h"^M
^M
PR_BEGIN_EXTERN_C^M
^M
/***********************************************************************^M
** DEFINES:     LL_MaxInt^M
**              LL_MinInt^M
**              LL_Zero^M
** DESCRIPTION:^M
**      Various interesting constants and static variable^M
**      initializer^M
***********************************************************************/^M
#if defined(HAVE_WATCOM_BUG_2)^M
PRInt64 __pascal __loadds __export^M
    LL_MaxInt(void);^M
PRInt64 __pascal __loadds __export^M
    LL_MinInt(void);^M
PRInt64 __pascal __loadds __export^M
    LL_Zero(void);^M
#else^M
NSPR_API(PRInt64) LL_MaxInt(void);^M
NSPR_API(PRInt64) LL_MinInt(void);^M
NSPR_API(PRInt64) LL_Zero(void);^M
#endif^M
^M
#define LL_MAXINT   LL_MaxInt()^M
#define LL_MININT   LL_MinInt()^M
#define LL_ZERO     LL_Zero()^M
^M
#if defined(HAVE_LONG_LONG)^M
^M
#if PR_BYTES_PER_LONG == 8^M
#define LL_INIT(hi, lo)  ((hi ## L << 32) + lo ## L)^M
#elif (defined(WIN32) || defined(WIN16)) && !defined(__GNUC__)^M
#define LL_INIT(hi, lo)  ((hi ## i64 << 32) + lo ## i64)^M
#else^M
#define LL_INIT(hi, lo)  ((hi ## LL << 32) + lo ## LL)^M
#endif^M
^M
/***********************************************************************^M
** MACROS:      LL_*^M
** DESCRIPTION:^M
**      The following macros define portable access to the 64 bit^M
**      math facilities.^M
**^M
***********************************************************************/^M
^M
/***********************************************************************^M
** MACROS:      LL_<relational operators>^M
**^M
**  LL_IS_ZERO        Test for zero^M
**  LL_EQ             Test for equality^M
**  LL_NE             Test for inequality^M
**  LL_GE_ZERO        Test for zero or positive^M
**  LL_CMP            Compare two values^M
***********************************************************************/^M
#define LL_IS_ZERO(a)       ((a) == 0)^M
#define LL_EQ(a, b)         ((a) == (b))^M
#define LL_NE(a, b)         ((a) != (b))^M
#define LL_GE_ZERO(a)       ((a) >= 0)^M
#define LL_CMP(a, op, b)    ((PRInt64)(a) op (PRInt64)(b))^M
#define LL_UCMP(a, op, b)   ((PRUint64)(a) op (PRUint64)(b))^M
^M
/***********************************************************************^M
** MACROS:      LL_<logical operators>^M
**^M
**  LL_AND            Logical and^M
**  LL_OR             Logical or^M
**  LL_XOR            Logical exclusion^M
**  LL_OR2            A disgusting deviation^M
**  LL_NOT            Negation (one's complement)^M
***********************************************************************/^M
#define LL_AND(r, a, b)        ((r) = (a) & (b))^M
#define LL_OR(r, a, b)        ((r) = (a) | (b))^M
#define LL_XOR(r, a, b)        ((r) = (a) ^ (b))^M
#define LL_OR2(r, a)        ((r) = (r) | (a))^M
#define LL_NOT(r, a)        ((r) = ~(a))^M
^M
/***********************************************************************^M
** MACROS:      LL_<mathematical operators>^M
**^M
**  LL_NEG            Negation (two's complement)^M
**  LL_ADD            Summation (two's complement)^M
**  LL_SUB            Difference (two's complement)^M
***********************************************************************/^M
#define LL_NEG(r, a)        ((r) = -(a))^M
#define LL_ADD(r, a, b)     ((r) = (a) + (b))^M
#define LL_SUB(r, a, b)     ((r) = (a) - (b))^M
^M
/***********************************************************************^M
** MACROS:      LL_<mathematical operators>^M
**^M
**  LL_MUL            Product (two's complement)^M
**  LL_DIV            Quotient (two's complement)^M
**  LL_MOD            Modulus (two's complement)^M
***********************************************************************/^M
#define LL_MUL(r, a, b)        ((r) = (a) * (b))^M
#define LL_DIV(r, a, b)        ((r) = (a) / (b))^M
#define LL_MOD(r, a, b)        ((r) = (a) % (b))^M
^M
/***********************************************************************^M
** MACROS:      LL_<shifting operators>^M
**^M
**  LL_SHL            Shift left [0..64] bits^M
**  LL_SHR            Shift right [0..64] bits with sign extension^M
**  LL_USHR           Unsigned shift right [0..64] bits^M
**  LL_ISHL           Signed shift left [0..64] bits^M
***********************************************************************/^M
#define LL_SHL(r, a, b)     ((r) = (PRInt64)(a) << (b))^M
#define LL_SHR(r, a, b)     ((r) = (PRInt64)(a) >> (b))^M
#define LL_USHR(r, a, b)    ((r) = (PRUint64)(a) >> (b))^M
#define LL_ISHL(r, a, b)    ((r) = (PRInt64)(a) << (b))^M
^M
/***********************************************************************^M
** MACROS:      LL_<conversion operators>^M
**^M
**  LL_L2I            Convert to signed 32 bit^M
**  LL_L2UI           Convert to unsigned 32 bit^M
**  LL_L2F            Convert to floating point^M
**  LL_L2D            Convert to floating point^M
**  LL_I2L            Convert signed to 64 bit^M
**  LL_UI2L           Convert unsigned to 64 bit^M
**  LL_F2L            Convert float to 64 bit^M
**  LL_D2L            Convert float to 64 bit^M
***********************************************************************/^M
#define LL_L2I(i, l)        ((i) = (PRInt32)(l))^M
#define LL_L2UI(ui, l)        ((ui) = (PRUint32)(l))^M
#define LL_L2F(f, l)        ((f) = (PRFloat64)(l))^M
#define LL_L2D(d, l)        ((d) = (PRFloat64)(l))^M
^M
#define LL_I2L(l, i)        ((l) = (PRInt64)(i))^M
#define LL_UI2L(l, ui)        ((l) = (PRInt64)(ui))^M
#define LL_F2L(l, f)        ((l) = (PRInt64)(f))^M
#define LL_D2L(l, d)        ((l) = (PRInt64)(d))^M
^M
/***********************************************************************^M
** MACROS:      LL_UDIVMOD^M
** DESCRIPTION:^M
**  Produce both a quotient and a remainder given an unsigned ^M
** INPUTS:      PRUint64 a: The dividend of the operation^M
**              PRUint64 b: The quotient of the operation^M
** OUTPUTS:     PRUint64 *qp: pointer to quotient^M
**              PRUint64 *rp: pointer to remainder^M
***********************************************************************/^M
#define LL_UDIVMOD(qp, rp, a, b) \^M
    (*(qp) = ((PRUint64)(a) / (b)), \^M
     *(rp) = ((PRUint64)(a) % (b)))^M
^M
#else  /* !HAVE_LONG_LONG */^M
^M
#ifdef IS_LITTLE_ENDIAN^M
#define LL_INIT(hi, lo) {PR_INT32(lo), PR_INT32(hi)}^M
#else^M
#define LL_INIT(hi, lo) {PR_INT32(hi), PR_INT32(lo)}^M
#endif^M
^M
#define LL_IS_ZERO(a)        (((a).hi == 0) && ((a).lo == 0))^M
#define LL_EQ(a, b)        (((a).hi == (b).hi) && ((a).lo == (b).lo))^M
#define LL_NE(a, b)        (((a).hi != (b).hi) || ((a).lo != (b).lo))^M
#define LL_GE_ZERO(a)        (((a).hi >> 31) == 0)^M
^M
#define LL_CMP(a, op, b)    (((a).hi == (b).hi) ? ((a).lo op (b).lo) : \^M
                 ((PRInt32)(a).hi op (PRInt32)(b).hi))^M
#define LL_UCMP(a, op, b)    (((a).hi == (b).hi) ? ((a).lo op (b).lo) : \^M
                 ((a).hi op (b).hi))^M
^M
#define LL_AND(r, a, b)        ((r).lo = (a).lo & (b).lo, \^M
                 (r).hi = (a).hi & (b).hi)^M
#define LL_OR(r, a, b)        ((r).lo = (a).lo | (b).lo, \^M
                 (r).hi = (a).hi | (b).hi)^M
#define LL_XOR(r, a, b)        ((r).lo = (a).lo ^ (b).lo, \^M
                 (r).hi = (a).hi ^ (b).hi)^M
#define LL_OR2(r, a)        ((r).lo = (r).lo | (a).lo, \^M
                 (r).hi = (r).hi | (a).hi)^M
#define LL_NOT(r, a)        ((r).lo = ~(a).lo, \^M
                 (r).hi = ~(a).hi)^M
^M
#define LL_NEG(r, a)        ((r).lo = -(PRInt32)(a).lo, \^M
                 (r).hi = -(PRInt32)(a).hi - ((r).lo != 0))^M
#define LL_ADD(r, a, b) { \^M
    PRInt64 _a, _b; \^M
    _a = a; _b = b; \^M
    (r).lo = _a.lo + _b.lo; \^M
    (r).hi = _a.hi + _b.hi + ((r).lo < _b.lo); \^M
}^M
^M
#define LL_SUB(r, a, b) { \^M
    PRInt64 _a, _b; \^M
    _a = a; _b = b; \^M
    (r).lo = _a.lo - _b.lo; \^M
    (r).hi = _a.hi - _b.hi - (_a.lo < _b.lo); \^M
}^M
^M
#define LL_MUL(r, a, b) { \^M
    PRInt64 _a, _b; \^M
    _a = a; _b = b; \^M
    LL_MUL32(r, _a.lo, _b.lo); \^M
    (r).hi += _a.hi * _b.lo + _a.lo * _b.hi; \^M
}^M
^M
#define _lo16(a)        ((a) & PR_BITMASK(16))^M
#define _hi16(a)        ((a) >> 16)^M
^M
#define LL_MUL32(r, a, b) { \^M
     PRUint32 _a1, _a0, _b1, _b0, _y0, _y1, _y2, _y3; \^M
     _a1 = _hi16(a), _a0 = _lo16(a); \^M
     _b1 = _hi16(b), _b0 = _lo16(b); \^M
     _y0 = _a0 * _b0; \^M
     _y1 = _a0 * _b1; \^M
     _y2 = _a1 * _b0; \^M
     _y3 = _a1 * _b1; \^M
     _y1 += _hi16(_y0);                         /* can't carry */ \^M
     _y1 += _y2;                                /* might carry */ \^M
     if (_y1 < _y2)    \^M
        _y3 += (PRUint32)(PR_BIT(16));  /* propagate */ \^M
     (r).lo = (_lo16(_y1) << 16) + _lo16(_y0); \^M
     (r).hi = _y3 + _hi16(_y1); \^M
}^M
^M
#define LL_UDIVMOD(qp, rp, a, b)    ll_udivmod(qp, rp, a, b)^M
^M
NSPR_API(void) ll_udivmod(PRUint64 *qp, PRUint64 *rp, PRUint64 a, PRUint64 b);^M
^M
#define LL_DIV(r, a, b) { \^M
    PRInt64 _a, _b; \^M
    PRUint32 _negative = (PRInt32)(a).hi < 0; \^M
    if (_negative) { \^M
    LL_NEG(_a, a); \^M
    } else { \^M
    _a = a; \^M
    } \^M
    if ((PRInt32)(b).hi < 0) { \^M
    _negative ^= 1; \^M
    LL_NEG(_b, b); \^M
    } else { \^M
    _b = b; \^M
    } \^M
    LL_UDIVMOD(&(r), 0, _a, _b); \^M
    if (_negative) \^M
    LL_NEG(r, r); \^M
}^M
^M
#define LL_MOD(r, a, b) { \^M
    PRInt64 _a, _b; \^M
    PRUint32 _negative = (PRInt32)(a).hi < 0; \^M
    if (_negative) { \^M
    LL_NEG(_a, a); \^M
    } else { \^M
    _a = a; \^M
    } \^M
    if ((PRInt32)(b).hi < 0) { \^M
    LL_NEG(_b, b); \^M
    } else { \^M
    _b = b; \^M
    } \^M
    LL_UDIVMOD(0, &(r), _a, _b); \^M
    if (_negative) \^M
    LL_NEG(r, r); \^M
}^M
^M
#define LL_SHL(r, a, b) { \^M
    if (b) { \^M
    PRInt64 _a; \^M
        _a = a; \^M
        if ((b) < 32) { \^M
        (r).lo = _a.lo << ((b) & 31); \^M
        (r).hi = (_a.hi << ((b) & 31)) | (_a.lo >> (32 - (b))); \^M
    } else { \^M
        (r).lo = 0; \^M
        (r).hi = _a.lo << ((b) & 31); \^M
    } \^M
    } else { \^M
    (r) = (a); \^M
    } \^M
}^M
^M
/* a is an PRInt32, b is PRInt32, r is PRInt64 */^M
#define LL_ISHL(r, a, b) { \^M
    if (b) { \^M
    PRInt64 _a; \^M
    _a.lo = (a); \^M
    _a.hi = 0; \^M
        if ((b) < 32) { \^M
        (r).lo = (a) << ((b) & 31); \^M
        (r).hi = ((a) >> (32 - (b))); \^M
    } else { \^M
        (r).lo = 0; \^M
        (r).hi = (a) << ((b) & 31); \^M
    } \^M
    } else { \^M
    (r).lo = (a); \^M
    (r).hi = 0; \^M
    } \^M
}^M
^M
#define LL_SHR(r, a, b) { \^M
    if (b) { \^M
    PRInt64 _a; \^M
        _a = a; \^M
    if ((b) < 32) { \^M
        (r).lo = (_a.hi << (32 - (b))) | (_a.lo >> ((b) & 31)); \^M
        (r).hi = (PRInt32)_a.hi >> ((b) & 31); \^M
    } else { \^M
        (r).lo = (PRInt32)_a.hi >> ((b) & 31); \^M
        (r).hi = (PRInt32)_a.hi >> 31; \^M
    } \^M
    } else { \^M
    (r) = (a); \^M
    } \^M
}^M
^M
#define LL_USHR(r, a, b) { \^M
    if (b) { \^M
    PRInt64 _a; \^M
        _a = a; \^M
    if ((b) < 32) { \^M
        (r).lo = (_a.hi << (32 - (b))) | (_a.lo >> ((b) & 31)); \^M
        (r).hi = _a.hi >> ((b) & 31); \^M
    } else { \^M
        (r).lo = _a.hi >> ((b) & 31); \^M
        (r).hi = 0; \^M
    } \^M
    } else { \^M
    (r) = (a); \^M
    } \^M
}^M
^M
#define LL_L2I(i, l)        ((i) = (l).lo)^M
#define LL_L2UI(ui, l)        ((ui) = (l).lo)^M
#define LL_L2F(f, l)        { double _d; LL_L2D(_d, l); (f) = (PRFloat64)_d; }^M
^M
#define LL_L2D(d, l) { \^M
    int _negative; \^M
    PRInt64 _absval; \^M
 \^M
    _negative = (l).hi >> 31; \^M
    if (_negative) { \^M
    LL_NEG(_absval, l); \^M
    } else { \^M
    _absval = l; \^M
    } \^M
    (d) = (double)_absval.hi * 4.294967296e9 + _absval.lo; \^M
    if (_negative) \^M
    (d) = -(d); \^M
}^M
^M
#define LL_I2L(l, i)        { PRInt32 _i = ((PRInt32)(i)) >> 31; (l).lo = (i); (l).hi = _i; }^M
#define LL_UI2L(l, ui)      ((l).lo = (ui), (l).hi = 0)^M
#define LL_F2L(l, f)        { double _d = (double)f; LL_D2L(l, _d); }^M
^M
#define LL_D2L(l, d) { \^M
    int _negative; \^M
    double _absval, _d_hi; \^M
    PRInt64 _lo_d; \^M
 \^M
    _negative = ((d) < 0); \^M
    _absval = _negative ? -(d) : (d); \^M
 \^M
    (l).hi = _absval / 4.294967296e9; \^M
    (l).lo = 0; \^M
    LL_L2D(_d_hi, l); \^M
    _absval -= _d_hi; \^M
    _lo_d.hi = 0; \^M
    if (_absval < 0) { \^M
    _lo_d.lo = -_absval; \^M
    LL_SUB(l, l, _lo_d); \^M
    } else { \^M
    _lo_d.lo = _absval; \^M
    LL_ADD(l, l, _lo_d); \^M
    } \^M
 \^M
    if (_negative) \^M
    LL_NEG(l, l); \^M
}^M
^M
#endif /* !HAVE_LONG_LONG */^M
^M
PR_END_EXTERN_C^M
^M
#endif /* prlong_h___ */^M
