/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1998-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions required by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
#ifndef prprf_h___^M
#define prprf_h___^M
^M
/*^M
** API for PR printf like routines. Supports the following formats^M
**	%d - decimal^M
**	%u - unsigned decimal^M
**	%x - unsigned hex^M
**	%X - unsigned uppercase hex^M
**	%o - unsigned octal^M
**	%hd, %hu, %hx, %hX, %ho - 16-bit versions of above^M
**	%ld, %lu, %lx, %lX, %lo - 32-bit versions of above^M
**	%lld, %llu, %llx, %llX, %llo - 64 bit versions of above^M
**	%s - string^M
**	%c - character^M
**	%p - pointer (deals with machine dependent pointer size)^M
**	%f - float^M
**	%g - float^M
*/^M
#include "prtypes.h"^M
#include "prio.h"^M
#include <stdio.h>^M
#include <stdarg.h>^M
^M
PR_BEGIN_EXTERN_C^M
^M
/*^M
** sprintf into a fixed size buffer. Guarantees that a NUL is at the end^M
** of the buffer. Returns the length of the written output, NOT including^M
** the NUL, or (PRUint32)-1 if an error occurs.^M
*/^M
NSPR_API(PRUint32) PR_snprintf(char *out, PRUint32 outlen, const char *fmt, ...);^M
^M
/*^M
** sprintf into a PR_MALLOC'd buffer. Return a pointer to the malloc'd^M
** buffer on success, NULL on failure. Call "PR_smprintf_free" to release^M
** the memory returned.^M
*/^M
NSPR_API(char*) PR_smprintf(const char *fmt, ...);^M
^M
/*^M
** Free the memory allocated, for the caller, by PR_smprintf^M
*/^M
NSPR_API(void) PR_smprintf_free(char *mem);^M
^M
/*^M
** "append" sprintf into a PR_MALLOC'd buffer. "last" is the last value of^M
** the PR_MALLOC'd buffer. sprintf will append data to the end of last,^M
** growing it as necessary using realloc. If last is NULL, PR_sprintf_append^M
** will allocate the initial string. The return value is the new value of^M
** last for subsequent calls, or NULL if there is a malloc failure.^M
*/^M
NSPR_API(char*) PR_sprintf_append(char *last, const char *fmt, ...);^M
^M
/*^M
** sprintf into a function. The function "f" is called with a string to^M
** place into the output. "arg" is an opaque pointer used by the stuff^M
** function to hold any state needed to do the storage of the output^M
** data. The return value is a count of the number of characters fed to^M
** the stuff function, or (PRUint32)-1 if an error occurs.^M
*/^M
typedef PRIntn (*PRStuffFunc)(void *arg, const char *s, PRUint32 slen);^M
^M
NSPR_API(PRUint32) PR_sxprintf(PRStuffFunc f, void *arg, const char *fmt, ...);^M
^M
/*^M
** fprintf to a PRFileDesc^M
*/^M
NSPR_API(PRUint32) PR_fprintf(struct PRFileDesc* fd, const char *fmt, ...);^M
^M
/*^M
** va_list forms of the above.^M
*/^M
NSPR_API(PRUint32) PR_vsnprintf(char *out, PRUint32 outlen, const char *fmt, va_list ap);^M
NSPR_API(char*) PR_vsmprintf(const char *fmt, va_list ap);^M
NSPR_API(char*) PR_vsprintf_append(char *last, const char *fmt, va_list ap);^M
NSPR_API(PRUint32) PR_vsxprintf(PRStuffFunc f, void *arg, const char *fmt, va_list ap);^M
NSPR_API(PRUint32) PR_vfprintf(struct PRFileDesc* fd, const char *fmt, va_list ap);^M
^M
/*^M
***************************************************************************^M
** FUNCTION: PR_sscanf^M
** DESCRIPTION:^M
**     PR_sscanf() scans the input character string, performs data^M
**     conversions, and stores the converted values in the data objects^M
**     pointed to by its arguments according to the format control^M
**     string.^M
**^M
**     PR_sscanf() behaves the same way as the sscanf() function in the^M
**     Standard C Library (stdio.h), with the following exceptions:^M
**     - PR_sscanf() handles the NSPR integer and floating point types,^M
**       such as PRInt16, PRInt32, PRInt64, and PRFloat64, whereas^M
**       sscanf() handles the standard C types like short, int, long,^M
**       and double.^M
**     - PR_sscanf() has no multibyte character support, while sscanf()^M
**       does.^M
** INPUTS:^M
**     const char *buf^M
**         a character string holding the input to scan^M
**     const char *fmt^M
**         the format control string for the conversions^M
**     ...^M
**         variable number of arguments, each of them is a pointer to^M
**         a data object in which the converted value will be stored^M
** OUTPUTS: none^M
** RETURNS: PRInt32^M
**     The number of values converted and stored.^M
** RESTRICTIONS:^M
**    Multibyte characters in 'buf' or 'fmt' are not allowed.^M
***************************************************************************^M
*/^M
^M
NSPR_API(PRInt32) PR_sscanf(const char *buf, const char *fmt, ...);^M
^M
PR_END_EXTERN_C^M
^M
#endif /* prprf_h___ */^M
