/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1999-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions required by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
/*^M
** prshm.h -- NSPR Shared Memory^M
**^M
** NSPR Named Shared Memory API provides a cross-platform named^M
** shared-memory interface. NSPR Named Shared Memory is modeled on^M
** similar constructs in Unix and Windows operating systems. Shared^M
** memory allows multiple processes to access one or more common shared^M
** memory regions, using it as an inter-process communication channel.^M
**^M
** Notes on Platform Independence:^M
**   NSPR Named Shared Memory is built on the native services offered^M
**   by most platforms. The NSPR Named Shared Memory API tries to^M
**   provide a least common denominator interface so that it works^M
**   across all supported platforms. To ensure that it works everywhere,^M
**   some platform considerations must be accomodated and the protocol^M
**   for using NSPR Shared Memory API must be observed.^M
**^M
** Protocol:^M
**   Multiple shared memories can be created using NSPR's Shared Memory^M
**   feature. For each named shared memory, as defined by the name^M
**   given in the PR_OpenSharedMemory() call, a protocol for using the^M
**   shared memory API is required to ensure desired behavior. Failing^M
**   to follow the protocol may yield unpredictable results.^M
**   ^M
**   PR_OpenSharedMemory() will create the shared memory segment, if it^M
**   does not already exist, or open a connection that the existing^M
**   shared memory segment if it already exists.^M
**   ^M
**   PR_AttachSharedMemory() should be called following^M
**   PR_OpenSharedMemory() to map the memory segment to an address in^M
**   the application's address space.^M
**   ^M
**   PR_AttachSharedMemory() may be called to re-map a shared memory^M
**   segment after detaching the same PRSharedMemory object. Be^M
**   sure to detach it when done.^M
**   ^M
**   PR_DetachSharedMemory() should be called to un-map the shared^M
**   memory segment from the application's address space.^M
**   ^M
**   PR_CloseSharedMemory() should be called when no further use of the^M
**   PRSharedMemory object is required within a process. Following a^M
**   call to  PR_CloseSharedMemory() the PRSharedMemory object is^M
**   invalid and cannot be reused.^M
**   ^M
**   PR_DeleteSharedMemory() should be called before process^M
**   termination. After calling PR_DeleteSharedMemory() any further use^M
**   of the shared memory associated with the name may cause^M
**   unpredictable results.^M
**   ^M
** Files:^M
**   The name passed to PR_OpenSharedMemory() should be a valid filename^M
**   for a unix platform. PR_OpenSharedMemory() creates file using the^M
**   name passed in. Some platforms may mangle the name before creating^M
**   the file and the shared memory.^M
**   ^M
**   The unix implementation may use SysV IPC shared memory, Posix^M
**   shared memory, or memory mapped files; the filename may used to^M
**   define the namespace. On Windows, the name is significant, but^M
**   there is no file associated with name.^M
**   ^M
**   No assumptions about the persistence of data in the named file^M
**   should be made. Depending on platform, the shared memory may be^M
**   mapped onto system paging space and be discarded at process^M
**   termination.^M
**   ^M
**   All names provided to PR_OpenSharedMemory() should be valid^M
**   filename syntax or name syntax for shared memory for the target^M
**   platform. Referenced directories should have permissions ^M
**   appropriate for writing.^M
**^M
** Limits:^M
**   Different platforms have limits on both the number and size of^M
**   shared memory resources. The default system limits on some^M
**   platforms may be smaller than your requirements. These limits may^M
**   be adjusted on some platforms either via boot-time options or by^M
**   setting the size of the system paging space to accomodate more^M
**   and/or larger shared memory segment(s).^M
**^M
** Security:^M
**   On unix platforms, depending on implementation, contents of the^M
**   backing store for the shared memory can be exposed via the file^M
**   system. Set permissions and or access controls at create and attach^M
**   time to ensure you get the desired security.^M
**^M
**   On windows platforms, no special security measures are provided.^M
**^M
** Example:^M
**   The test case pr/tests/nameshm1.c provides an example of use as^M
**   well as testing the operation of NSPR's Named Shared Memory.^M
**^M
** lth. 18-Aug-1999.^M
*/^M
^M
#ifndef prshm_h___^M
#define prshm_h___^M
^M
#include "prtypes.h"^M
#include "prio.h"^M
^M
PR_BEGIN_EXTERN_C^M
^M
/*^M
** Declare opaque type PRSharedMemory.^M
*/^M
typedef struct PRSharedMemory PRSharedMemory;^M
^M
/*^M
** FUNCTION: PR_OpenSharedMemory()^M
**^M
** DESCRIPTION:^M
**   PR_OpenSharedMemory() creates a new shared-memory segment or^M
**   associates a previously created memory segment with name.^M
**^M
**   When parameter create is (PR_SHM_EXCL | PR_SHM_CREATE) and the^M
**   shared memory already exists, the function returns NULL with the^M
**   error set to PR_FILE_EXISTS_ERROR.^M
**^M
**   When parameter create is PR_SHM_CREATE and the shared memory^M
**   already exists, a handle to that memory segment is returned. If^M
**   the segment does not exist, it is created and a pointer to the^M
**   related PRSharedMemory structure is returned.^M
**^M
**   When parameter create is 0, and the shared memory exists, a^M
**   pointer to a PRSharedMemory is returned. If the shared memory does^M
**   not exist, NULL is returned with the error set to^M
**   PR_FILE_NOT_FOUND_ERROR.^M
**^M
** INPUTS:^M
**   name -- the name the shared-memory segment is known as.^M
**   size -- the size of the shared memory segment. ^M
**   flags -- Options for creating the shared memory^M
**   mode -- Same as is passed to PR_Open()^M
**^M
** OUTPUTS: ^M
**   The shared memory is allocated.^M
**^M
** RETURNS: Pointer to opaque structure PRSharedMemory or NULL.^M
**   NULL is returned on error. The reason for the error can be^M
**   retrieved via PR_GetError() and PR_GetOSError();^M
**^M
*/^M
NSPR_API( PRSharedMemory * )^M
    PR_OpenSharedMemory(^M
        const char *name,^M
        PRSize      size,^M
        PRIntn      flags,^M
        PRIntn      mode^M
);^M
/* Define values for PR_OpenShareMemory(...,create) */^M
#define PR_SHM_CREATE 0x1  /* create if not exist */^M
#define PR_SHM_EXCL   0x2  /* fail if already exists */^M
^M
/*^M
** FUNCTION: PR_AttachSharedMemory()^M
**^M
** DESCRIPTION:^M
** PR_AttachSharedMemory() maps the shared-memory described by^M
** shm to the current process. ^M
**^M
** INPUTS: ^M
**   shm -- The handle returned from PR_OpenSharedMemory().^M
**   flags -- options for mapping the shared memory.^M
**   PR_SHM_READONLY causes the memory to be attached ^M
**   read-only.^M
**^M
** OUTPUTS:^M
**   On success, the shared memory segment represented by shm is mapped^M
**   into the process' address space.^M
**^M
** RETURNS: Address where shared memory is mapped, or NULL.^M
**   NULL is returned on error. The reason for the error can be^M
**   retrieved via PR_GetError() and PR_GetOSError();^M
**^M
**^M
*/^M
NSPR_API( void * )^M
    PR_AttachSharedMemory(^M
        PRSharedMemory *shm,^M
        PRIntn  flags^M
);^M
/* Define values for PR_AttachSharedMemory(...,flags) */ ^M
#define PR_SHM_READONLY 0x01^M
^M
/*^M
** FUNCTION: PR_DetachSharedMemory()^M
**^M
** DESCRIPTION:^M
**   PR_DetachSharedMemory() detaches the shared-memory described^M
**   by shm. ^M
**^M
** INPUTS: ^M
**   shm -- The handle returned from PR_OpenSharedMemory().^M
**   addr -- The address at which the memory was attached.^M
**^M
** OUTPUTS:^M
**   The shared memory mapped to an address via a previous call to^M
**   PR_AttachSharedMemory() is unmapped.^M
**^M
** RETURNS: PRStatus^M
**^M
*/^M
NSPR_API( PRStatus )^M
    PR_DetachSharedMemory(^M
        PRSharedMemory *shm,^M
        void  *addr^M
);^M
^M
/*^M
** FUNCTION: PR_CloseSharedMemory()^M
**^M
** DESCRIPTION:^M
**   PR_CloseSharedMemory() closes the shared-memory described by^M
**   shm.^M
** ^M
** INPUTS:^M
**   shm -- The handle returned from PR_OpenSharedMemory().^M
**^M
** OUTPUTS:^M
**   the shared memory represented by shm is closed^M
**^M
** RETURNS: PRStatus^M
**^M
*/^M
NSPR_API( PRStatus )^M
    PR_CloseSharedMemory(^M
        PRSharedMemory *shm^M
);^M
^M
/*^M
** FUNCTION: PR_DeleteSharedMemory()^M
**^M
** DESCRIPTION:^M
**   The shared memory resource represented by name is released.^M
**^M
** INPUTS:^M
**   name -- the name the shared-memory segment^M
**^M
** OUTPUTS:^M
**   depending on platform, resources may be returned to the underlying^M
**   operating system.^M
**^M
** RETURNS: PRStatus^M
**^M
*/^M
NSPR_API( PRStatus )^M
    PR_DeleteSharedMemory( ^M
        const char *name^M
);^M
^M
PR_END_EXTERN_C^M
^M
#endif /* prshm_h___ */^M
