/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1998-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions required by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
/*^M
 *----------------------------------------------------------------------^M
 *^M
 * prtime.h --^M
 *^M
 *     NSPR date and time functions^M
 *^M
 *-----------------------------------------------------------------------^M
 */^M
^M
#ifndef prtime_h___^M
#define prtime_h___^M
^M
#include "prlong.h"^M
^M
PR_BEGIN_EXTERN_C^M
^M
/**********************************************************************/^M
/************************* TYPES AND CONSTANTS ************************/^M
/**********************************************************************/^M
^M
#define PR_MSEC_PER_SEC		1000UL^M
#define PR_USEC_PER_SEC		1000000UL^M
#define PR_NSEC_PER_SEC		1000000000UL^M
#define PR_USEC_PER_MSEC	1000UL^M
#define PR_NSEC_PER_MSEC	1000000UL^M
^M
/*^M
 * PRTime --^M
 *^M
 *     NSPR represents basic time as 64-bit signed integers relative^M
 *     to midnight (00:00:00), January 1, 1970 Greenwich Mean Time (GMT).^M
 *     (GMT is also known as Coordinated Universal Time, UTC.)^M
 *     The units of time are in microseconds. Negative times are allowed^M
 *     to represent times prior to the January 1970 epoch. Such values are^M
 *     intended to be exported to other systems or converted to human^M
 *     readable form.^M
 *^M
 *     Notes on porting: PRTime corresponds to time_t in ANSI C.  NSPR 1.0^M
 *     simply uses PRInt64.^M
 */^M
^M
typedef PRInt64 PRTime;^M
^M
/*^M
 * Time zone and daylight saving time corrections applied to GMT to^M
 * obtain the local time of some geographic location^M
 */^M
^M
typedef struct PRTimeParameters {^M
    PRInt32 tp_gmt_offset;     /* the offset from GMT in seconds */^M
    PRInt32 tp_dst_offset;     /* contribution of DST in seconds */^M
} PRTimeParameters;^M
^M
/*^M
 * PRExplodedTime --^M
 *^M
 *     Time broken down into human-readable components such as year, month,^M
 *     day, hour, minute, second, and microsecond.  Time zone and daylight^M
 *     saving time corrections may be applied.  If they are applied, the^M
 *     offsets from the GMT must be saved in the 'tm_params' field so that^M
 *     all the information is available to reconstruct GMT.^M
 *^M
 *     Notes on porting: PRExplodedTime corrresponds to struct tm in^M
 *     ANSI C, with the following differences:^M
 *       - an additional field tm_usec;^M
 *       - replacing tm_isdst by tm_params;^M
 *       - the month field is spelled tm_month, not tm_mon;^M
 *       - we use absolute year, AD, not the year since 1900.^M
 *     The corresponding type in NSPR 1.0 is called PRTime.  Below is^M
 *     a table of date/time type correspondence in the three APIs:^M
 *         API          time since epoch          time in components^M
 *       ANSI C             time_t                  struct tm^M
 *       NSPR 1.0           PRInt64                   PRTime^M
 *       NSPR 2.0           PRTime                  PRExplodedTime^M
 */^M
^M
typedef struct PRExplodedTime {^M
    PRInt32 tm_usec;		    /* microseconds past tm_sec (0-99999)  */^M
    PRInt32 tm_sec;             /* seconds past tm_min (0-61, accomodating^M
                                   up to two leap seconds) */	^M
    PRInt32 tm_min;             /* minutes past tm_hour (0-59) */^M
    PRInt32 tm_hour;            /* hours past tm_day (0-23) */^M
    PRInt32 tm_mday;            /* days past tm_mon (1-31, note that it^M
				                starts from 1) */^M
    PRInt32 tm_month;           /* months past tm_year (0-11, Jan = 0) */^M
    PRInt16 tm_year;            /* absolute year, AD (note that we do not^M
				                count from 1900) */^M
^M
    PRInt8 tm_wday;		        /* calculated day of the week^M
				                (0-6, Sun = 0) */^M
    PRInt16 tm_yday;            /* calculated day of the year ^M
				                (0-365, Jan 1 = 0) */^M
^M
    PRTimeParameters tm_params;  /* time parameters used by conversion */^M
} PRExplodedTime;^M
^M
/*^M
 * PRTimeParamFn --^M
 *^M
 *     A function of PRTimeParamFn type returns the time zone and^M
 *     daylight saving time corrections for some geographic location,^M
 *     given the current time in GMT.  The input argument gmt should^M
 *     point to a PRExplodedTime that is in GMT, i.e., whose^M
 *     tm_params contains all 0's.^M
 *^M
 *     For any time zone other than GMT, the computation is intended to^M
 *     consist of two steps:^M
 *       - Figure out the time zone correction, tp_gmt_offset.  This number^M
 *         usually depends on the geographic location only.  But it may^M
 *         also depend on the current time.  For example, all of China^M
 *         is one time zone right now.  But this situation may change^M
 *         in the future.^M
 *       - Figure out the daylight saving time correction, tp_dst_offset.^M
 *         This number depends on both the geographic location and the^M
 *         current time.  Most of the DST rules are expressed in local^M
 *         current time.  If so, one should apply the time zone correction^M
 *         to GMT before applying the DST rules.^M
 */^M
^M
typedef PRTimeParameters (PR_CALLBACK_DECL *PRTimeParamFn)(const PRExplodedTime *gmt);^M
^M
/**********************************************************************/^M
/****************************** FUNCTIONS *****************************/^M
/**********************************************************************/^M
^M
/*^M
 * The PR_Now routine returns the current time relative to the^M
 * epoch, midnight, January 1, 1970 UTC. The units of the returned^M
 * value are microseconds since the epoch.^M
 *^M
 * The values returned are not guaranteed to advance in a linear fashion^M
 * due to the application of time correction protocols which synchronize^M
 * computer clocks to some external time source. Consequently it should^M
 * not be depended on for interval timing.^M
 *^M
 * The implementation is machine dependent.^M
 * Cf. time_t time(time_t *tp) in ANSI C.^M
 */^M
#if defined(HAVE_WATCOM_BUG_2)^M
PRTime __pascal __export __loadds^M
#else^M
NSPR_API(PRTime) ^M
#endif^M
PR_Now(void);^M
^M
/*^M
 * Expand time binding it to time parameters provided by PRTimeParamFn.^M
 * The calculation is envisoned to proceed in the following steps:^M
 *   - From given PRTime, calculate PRExplodedTime in GMT^M
 *   - Apply the given PRTimeParamFn to the GMT that we just calculated^M
 *     to obtain PRTimeParameters.^M
 *   - Add the PRTimeParameters offsets to GMT to get the local time^M
 *     as PRExplodedTime.^M
 */^M
^M
NSPR_API(void) PR_ExplodeTime(^M
    PRTime usecs, PRTimeParamFn params, PRExplodedTime *exploded);^M
^M
/* Reverse operation of PR_ExplodeTime */^M
#if defined(HAVE_WATCOM_BUG_2)^M
PRTime __pascal __export __loadds^M
#else^M
NSPR_API(PRTime) ^M
#endif^M
PR_ImplodeTime(const PRExplodedTime *exploded);^M
^M
/*^M
 * Adjust exploded time to normalize field overflows after manipulation.^M
 * Note that the following fields of PRExplodedTime should not be^M
 * manipulated:^M
 *   - tm_month and tm_year: because the number of days in a month and^M
 *     number of days in a year are not constant, it is ambiguous to^M
 *     manipulate the month and year fields, although one may be tempted^M
 *     to.  For example, what does "a month from January 31st" mean?^M
 *   - tm_wday and tm_yday: these fields are calculated by NSPR.  Users^M
 *     should treat them as "read-only".^M
 */^M
^M
NSPR_API(void) PR_NormalizeTime(^M
    PRExplodedTime *exploded, PRTimeParamFn params);^M
^M
/**********************************************************************/^M
/*********************** TIME PARAMETER FUNCTIONS *********************/^M
/**********************************************************************/^M
^M
/* Time parameters that suit current host machine */^M
NSPR_API(PRTimeParameters) PR_LocalTimeParameters(const PRExplodedTime *gmt);^M
^M
/* Time parameters that represent Greenwich Mean Time */^M
NSPR_API(PRTimeParameters) PR_GMTParameters(const PRExplodedTime *gmt);^M
^M
/*^M
 * Time parameters that represent the US Pacific Time Zone, with the^M
 * current daylight saving time rules (for testing only)^M
 */^M
NSPR_API(PRTimeParameters) PR_USPacificTimeParameters(const PRExplodedTime *gmt);^M
^M
/*^M
 * This parses a time/date string into a PRTime^M
 * (microseconds after "1-Jan-1970 00:00:00 GMT").^M
 * It returns PR_SUCCESS on success, and PR_FAILURE^M
 * if the time/date string can't be parsed.^M
 *^M
 * Many formats are handled, including:^M
 *^M
 *   14 Apr 89 03:20:12^M
 *   14 Apr 89 03:20 GMT^M
 *   Fri, 17 Mar 89 4:01:33^M
 *   Fri, 17 Mar 89 4:01 GMT^M
 *   Mon Jan 16 16:12 PDT 1989^M
 *   Mon Jan 16 16:12 +0130 1989^M
 *   6 May 1992 16:41-JST (Wednesday)^M
 *   22-AUG-1993 10:59:12.82^M
 *   22-AUG-1993 10:59pm^M
 *   22-AUG-1993 12:59am^M
 *   22-AUG-1993 12:59 PM^M
 *   Friday, August 04, 1995 3:54 PM^M
 *   06/21/95 04:24:34 PM^M
 *   20/06/95 21:07^M
 *   95-06-08 19:32:48 EDT^M
 *^M
 * If the input string doesn't contain a description of the timezone,^M
 * we consult the `default_to_gmt' to decide whether the string should^M
 * be interpreted relative to the local time zone (PR_FALSE) or GMT (PR_TRUE).^M
 * The correct value for this argument depends on what standard specified^M
 * the time string which you are parsing.^M
 */^M
^M
NSPR_API(PRStatus) PR_ParseTimeString (^M
	const char *string,^M
	PRBool default_to_gmt,^M
	PRTime *result);^M
^M
/*^M
 * FIXME: should we also have a formatting function, such as asctime, ctime,^M
 * and strftime in standard C library?  But this would involve^M
 * internationalization issues.  Might want to provide a US English version.^M
 */^M
^M
/**********************************************************************/^M
/*********************** OLD COMPATIBILITYFUNCTIONS *******************/^M
/**********************************************************************/^M
#ifndef NO_NSPR_10_SUPPORT^M
^M
/* Format a time value into a buffer. Same semantics as strftime() */^M
NSPR_API(PRUint32) PR_FormatTime(char *buf, int buflen, const char *fmt, ^M
                                           const PRExplodedTime *tm);^M
^M
/* Format a time value into a buffer. Time is always in US English format, regardless^M
 * of locale setting.^M
 */^M
NSPR_API(PRUint32)^M
PR_FormatTimeUSEnglish( char* buf, PRUint32 bufSize,^M
                        const char* format, const PRExplodedTime* tm );^M
^M
#endif /* NO_NSPR_10_SUPPORT */^M
^M
PR_END_EXTERN_C^M
^M
#endif /* prtime_h___ */^M
